@@ -20,16 +20,20 @@
 
 #include "cluster.h"
 
+#include <config/config_util.h>
+
 #include <algorithm>
-#include <cassert>
 #include <cstring>
+#include <fstream>
 #include <memory>
 
 #include "commands/redis_cmd.h"
+#include "fmt/format.h"
 #include "parse_util.h"
 #include "replication.h"
 #include "server/server.h"
-#include "util.h"
+#include "string_util.h"
+#include "time_util.h"
 
 const char *errInvalidNodeID = "Invalid cluster node id";
 const char *errInvalidSlotID = "Invalid slot id";
@@ -77,7 +81,7 @@ bool Cluster::SubCommandIsExecExclusive(const std::string &subcommand) {
 
 Status Cluster::SetNodeId(const std::string &node_id) {
   if (node_id.size() != kClusterNodeIdLen) {
-    return Status(Status::ClusterInvalidInfo, errInvalidNodeID);
+    return {Status::ClusterInvalidInfo, errInvalidNodeID};
   }
 
   myid_ = node_id;
@@ -89,7 +93,7 @@ Status Cluster::SetNodeId(const std::string &node_id) {
   }
 
   // Set replication relationship
-  if (myself_ != nullptr) SetMasterSlaveRepl();
+  if (myself_) return SetMasterSlaveRepl();
 
   return Status::OK();
 }
@@ -108,22 +112,25 @@ Status Cluster::SetNodeId(const std::string &node_id) {
 Status Cluster::SetSlot(int slot, const std::string &node_id, int64_t new_version) {
   // Parameters check
   if (new_version <= 0 || new_version != version_ + 1) {
-    return Status(Status::NotOK, errInvalidClusterVersion);
+    return {Status::NotOK, errInvalidClusterVersion};
   }
+
   if (!IsValidSlot(slot)) {
-    return Status(Status::NotOK, errInvalidSlotID);
+    return {Status::NotOK, errInvalidSlotID};
   }
+
   if (node_id.size() != kClusterNodeIdLen) {
-    return Status(Status::NotOK, errInvalidNodeID);
+    return {Status::NotOK, errInvalidNodeID};
   }
 
   // Get the node which we want to assign a slot into it
   std::shared_ptr<ClusterNode> to_assign_node = nodes_[node_id];
   if (to_assign_node == nullptr) {
-    return Status(Status::NotOK, "No this node in the cluster");
+    return {Status::NotOK, "No this node in the cluster"};
   }
+
   if (to_assign_node->role_ != kClusterMaster) {
-    return Status(Status::NotOK, errNoMasterNode);
+    return {Status::NotOK, errNoMasterNode};
   }
 
   // Update version
@@ -143,12 +150,12 @@ Status Cluster::SetSlot(int slot, const std::string &node_id, int64_t new_versio
   // Clear data of migrated slot or record of imported slot
   if (old_node == myself_ && old_node != to_assign_node) {
     // If slot is migrated from this node
-    if (migrated_slots_.count(slot)) {
+    if (migrated_slots_.count(slot) > 0) {
       svr_->slot_migrate_->ClearKeysOfSlot(kDefaultNamespace, slot);
       migrated_slots_.erase(slot);
     }
     // If slot is imported into this node
-    if (imported_slots_.count(slot)) {
+    if (imported_slots_.count(slot) > 0) {
       imported_slots_.erase(slot);
     }
   }
@@ -159,13 +166,14 @@ Status Cluster::SetSlot(int slot, const std::string &node_id, int64_t new_versio
 // cluster setnodes $all_nodes_info $version $force
 // one line of $all_nodes: $node_id $host $port $role $master_node_id $slot_range
 Status Cluster::SetClusterNodes(const std::string &nodes_str, int64_t version, bool force) {
-  if (version < 0) return Status(Status::NotOK, errInvalidClusterVersion);
+  if (version < 0) return {Status::NotOK, errInvalidClusterVersion};
 
-  if (force == false) {
+  if (!force) {
     // Low version wants to reset current version
     if (version_ > version) {
-      return Status(Status::NotOK, errInvalidClusterVersion);
+      return {Status::NotOK, errInvalidClusterVersion};
     }
+
     // The same version, it is not needed to update
     if (version_ == version) return Status::OK();
   }
@@ -212,7 +220,12 @@ Status Cluster::SetClusterNodes(const std::string &nodes_str, int64_t version, b
   }
 
   // Set replication relationship
-  if (myself_ != nullptr) SetMasterSlaveRepl();
+  if (myself_) {
+    s = SetMasterSlaveRepl();
+    if (!s.IsOK()) {
+      return s.Prefixed("failed to set master-replica replication");
+    }
+  }
 
   // Clear data of migrated slots
   if (!migrated_slots_.empty()) {
@@ -230,33 +243,40 @@ Status Cluster::SetClusterNodes(const std::string &nodes_str, int64_t version, b
 }
 
 // Set replication relationship by cluster topology setting
-void Cluster::SetMasterSlaveRepl() {
-  if (svr_ == nullptr) return;
-  if (myself_ == nullptr) return;
+Status Cluster::SetMasterSlaveRepl() {
+  if (!svr_) return Status::OK();
+
+  if (!myself_) return Status::OK();
 
   if (myself_->role_ == kClusterMaster) {
     // Master mode
-    svr_->RemoveMaster();
+    auto s = svr_->RemoveMaster();
+    if (!s.IsOK()) {
+      return s.Prefixed("failed to remove master");
+    }
     LOG(INFO) << "MASTER MODE enabled by cluster topology setting";
   } else if (nodes_.find(myself_->master_id_) != nodes_.end()) {
-    // Slave mode and master node is existing
+    // Replica mode and master node is existing
     std::shared_ptr<ClusterNode> master = nodes_[myself_->master_id_];
-    Status s = svr_->AddMaster(master->host_, master->port_, false);
-    if (s.IsOK()) {
-      LOG(INFO) << "SLAVE OF " << master->host_ << ":" << master->port_ << " enabled by cluster topology setting";
-    } else {
+    auto s = svr_->AddMaster(master->host_, master->port_, false);
+    if (!s.IsOK()) {
       LOG(WARNING) << "SLAVE OF " << master->host_ << ":" << master->port_
-                   << " enabled by cluster topology setting, encounter error: " << s.Msg();
+                   << " wasn't enabled by cluster topology setting, encounter error: " << s.Msg();
+      return s.Prefixed("failed to add master");
     }
+    LOG(INFO) << "SLAVE OF " << master->host_ << ":" << master->port_ << " enabled by cluster topology setting";
   }
+
+  return Status::OK();
 }
 
 bool Cluster::IsNotMaster() { return myself_ == nullptr || myself_->role_ != kClusterMaster || svr_->IsSlave(); }
 
 Status Cluster::SetSlotMigrated(int slot, const std::string &ip_port) {
   if (!IsValidSlot(slot)) {
-    return Status(Status::NotOK, errSlotOutOfRange);
+    return {Status::NotOK, errSlotOutOfRange};
   }
+
   // It is called by slot-migrating thread which is an asynchronous thread.
   // Therefore, it should be locked when a record is added to 'migrated_slots_'
   // which will be accessed when executing commands.
@@ -267,8 +287,9 @@ Status Cluster::SetSlotMigrated(int slot, const std::string &ip_port) {
 
 Status Cluster::SetSlotImported(int slot) {
   if (!IsValidSlot(slot)) {
-    return Status(Status::NotOK, errSlotOutOfRange);
+    return {Status::NotOK, errSlotOutOfRange};
   }
+
   // It is called by command 'cluster import'. When executing the command, the
   // exclusive lock has been locked. Therefore, it can't be locked again.
   imported_slots_.insert(slot);
@@ -277,22 +298,27 @@ Status Cluster::SetSlotImported(int slot) {
 
 Status Cluster::MigrateSlot(int slot, const std::string &dst_node_id) {
   if (nodes_.find(dst_node_id) == nodes_.end()) {
-    return Status(Status::NotOK, "Can't find the destination node id");
+    return {Status::NotOK, "Can't find the destination node id"};
   }
+
   if (!IsValidSlot(slot)) {
-    return Status(Status::NotOK, errSlotOutOfRange);
+    return {Status::NotOK, errSlotOutOfRange};
   }
+
   if (slots_nodes_[slot] != myself_) {
-    return Status(Status::NotOK, "Can't migrate slot which doesn't belong to me");
+    return {Status::NotOK, "Can't migrate slot which doesn't belong to me"};
   }
+
   if (IsNotMaster()) {
-    return Status(Status::NotOK, "Slave can't migrate slot");
+    return {Status::NotOK, "Slave can't migrate slot"};
   }
+
   if (nodes_[dst_node_id]->role_ != kClusterMaster) {
-    return Status(Status::NotOK, "Can't migrate slot to a slave");
+    return {Status::NotOK, "Can't migrate slot to a slave"};
   }
+
   if (nodes_[dst_node_id] == myself_) {
-    return Status(Status::NotOK, "Can't migrate slot to myself");
+    return {Status::NotOK, "Can't migrate slot to myself"};
   }
 
   const auto dst = nodes_[dst_node_id];
@@ -304,22 +330,26 @@ Status Cluster::MigrateSlot(int slot, const std::string &dst_node_id) {
 
 Status Cluster::ImportSlot(Redis::Connection *conn, int slot, int state) {
   if (IsNotMaster()) {
-    return Status(Status::NotOK, "Slave can't import slot");
+    return {Status::NotOK, "Slave can't import slot"};
   }
+
   if (!IsValidSlot(slot)) {
-    return Status(Status::NotOK, errSlotOutOfRange);
+    return {Status::NotOK, errSlotOutOfRange};
   }
 
   switch (state) {
     case kImportStart:
       if (!svr_->slot_import_->Start(conn->GetFD(), slot)) {
-        return Status(Status::NotOK, "Can't start importing slot " + std::to_string(slot));
+        return {Status::NotOK, fmt::format("Can't start importing slot {}", slot)};
       }
+
       // Set link importing
       conn->SetImporting();
       myself_->importing_slot_ = slot;
       // Set link error callback
-      conn->close_cb_ = std::bind(&SlotImport::StopForLinkError, svr_->slot_import_, conn->GetFD());
+      conn->close_cb_ = [object_ptr = svr_->slot_import_, capture_fd = conn->GetFD()](int fd) {
+        object_ptr->StopForLinkError(capture_fd);
+      };
       // Stop forbidding writing slot to accept write commands
       if (slot == svr_->slot_migrate_->GetForbiddenSlot()) svr_->slot_migrate_->ReleaseForbiddenSlot();
       LOG(INFO) << "[import] Start importing slot " << slot;
@@ -328,28 +358,32 @@ Status Cluster::ImportSlot(Redis::Connection *conn, int slot, int state) {
       if (!svr_->slot_import_->Success(slot)) {
         LOG(ERROR) << "[import] Failed to set slot importing success, maybe slot is wrong"
                    << ", received slot: " << slot << ", current slot: " << svr_->slot_import_->GetSlot();
-        return Status(Status::NotOK, "Failed to set slot " + std::to_string(slot) + " importing success");
+        return {Status::NotOK, fmt::format("Failed to set slot {} importing success", slot)};
       }
+
       LOG(INFO) << "[import] Succeed to import slot " << slot;
       break;
     case kImportFailed:
       if (!svr_->slot_import_->Fail(slot)) {
         LOG(ERROR) << "[import] Failed to set slot importing error, maybe slot is wrong"
                    << ", received slot: " << slot << ", current slot: " << svr_->slot_import_->GetSlot();
-        return Status(Status::NotOK, "Failed to set slot " + std::to_string(slot) + " importing error");
+        return {Status::NotOK, fmt::format("Failed to set slot {} importing error", slot)};
       }
+
       LOG(INFO) << "[import] Failed to import slot " << slot;
       break;
     default:
-      return Status(Status::NotOK, errInvalidImportState);
+      return {Status::NotOK, errInvalidImportState};
   }
+
   return Status::OK();
 }
 
 Status Cluster::GetClusterInfo(std::string *cluster_infos) {
   if (version_ < 0) {
-    return Status(Status::ClusterDown, errClusterNoInitialized);
+    return {Status::ClusterDown, errClusterNoInitialized};
   }
+
   cluster_infos->clear();
 
   int ok_slot = 0;
@@ -405,8 +439,9 @@ Status Cluster::GetClusterInfo(std::string *cluster_infos) {
 //          ... continued until done
 Status Cluster::GetSlotsInfo(std::vector<SlotInfo> *slots_infos) {
   if (version_ < 0) {
-    return Status(Status::ClusterDown, errClusterNoInitialized);
+    return {Status::ClusterDown, errClusterNoInitialized};
   }
+
   slots_infos->clear();
 
   int start = -1;
@@ -427,6 +462,7 @@ Status Cluster::GetSlotsInfo(std::vector<SlotInfo> *slots_infos) {
       start = i;
     }
   }
+
   return Status::OK();
 }
 
@@ -438,6 +474,7 @@ SlotInfo Cluster::GenSlotNodeInfo(int start, int end, const std::shared_ptr<Clus
     if (nodes_.find(id) == nodes_.end()) continue;
     vn.push_back({nodes_[id]->host_, nodes_[id]->port_, nodes_[id]->id_});
   }
+
   return {start, end, vn};
 }
 
@@ -445,7 +482,7 @@ SlotInfo Cluster::GenSlotNodeInfo(int start, int end, const std::shared_ptr<Clus
 // $version $connected $slot_range
 Status Cluster::GetClusterNodes(std::string *nodes_str) {
   if (version_ < 0) {
-    return Status(Status::ClusterDown, errClusterNoInitialized);
+    return {Status::ClusterDown, errClusterNoInitialized};
   }
 
   *nodes_str = GenNodesDescription();
@@ -453,8 +490,46 @@ Status Cluster::GetClusterNodes(std::string *nodes_str) {
 }
 
 std::string Cluster::GenNodesDescription() {
-  // Generate slots info firstly
+  UpdateSlotsInfo();
+
+  auto now = Util::GetTimeStampMS();
+  std::string nodes_desc;
+  for (const auto &item : nodes_) {
+    const std::shared_ptr<ClusterNode> n = item.second;
+
+    std::string node_str;
+    // ID, host, port
+    node_str.append(n->id_ + " ");
+    node_str.append(fmt::format("{}:{}@{} ", n->host_, n->port_, n->port_ + kClusterPortIncr));
+
+    // Flags
+    if (n->id_ == myid_) node_str.append("myself,");
+    if (n->role_ == kClusterMaster) {
+      node_str.append("master - ");
+    } else {
+      node_str.append("slave " + n->master_id_ + " ");
+    }
+
+    // Ping sent, pong received, config epoch, link status
+    node_str.append(fmt::format("{} {} {} connected", now - 1, now, version_));
+
+    if (n->role_ == kClusterMaster && n->slots_info_.size() > 0) {
+      node_str.append(" " + n->slots_info_);
+    }
+
+    nodes_desc.append(node_str + "\n");
+  }
+  return nodes_desc;
+}
+
+void Cluster::UpdateSlotsInfo() {
   int start = -1;
+  // reset the previous slots info
+  for (const auto &item : nodes_) {
+    const std::shared_ptr<ClusterNode> &n = item.second;
+    n->slots_info_.clear();
+  }
+
   std::shared_ptr<ClusterNode> n = nullptr;
   for (int i = 0; i <= kClusterSlots; i++) {
     // Find start node and slot id
@@ -467,9 +542,9 @@ std::string Cluster::GenNodesDescription() {
     // Generate slots info when occur different node with start or end of slot
     if (i == kClusterSlots || n != slots_nodes_[i]) {
       if (start == i - 1) {
-        n->slots_info_ += std::to_string(start) + " ";
+        n->slots_info_ += fmt::format("{} ", start);
       } else {
-        n->slots_info_ += std::to_string(start) + "-" + std::to_string(i - 1) + " ";
+        n->slots_info_ += fmt::format("{}-{} ", start, i - 1);
       }
       if (i == kClusterSlots) break;
       n = slots_nodes_[i];
@@ -477,60 +552,121 @@ std::string Cluster::GenNodesDescription() {
     }
   }
 
-  std::string nodes_desc;
   for (const auto &item : nodes_) {
     const std::shared_ptr<ClusterNode> n = item.second;
+    if (n->slots_info_.size() > 0) n->slots_info_.pop_back();  // Remove last space
+  }
+}
 
+std::string Cluster::GenNodesInfo() {
+  UpdateSlotsInfo();
+
+  std::string nodes_info;
+  for (const auto &item : nodes_) {
+    const std::shared_ptr<ClusterNode> &n = item.second;
     std::string node_str;
-    // ID, host, port
+    node_str.append("node ");
+    // ID
     node_str.append(n->id_ + " ");
-    node_str.append(n->host_ + ":" + std::to_string(n->port_) + "@" + std::to_string(n->port_ + kClusterPortIncr) +
-                    " ");
+    // Host + Port
+    node_str.append(fmt::format("{} {} ", n->host_, n->port_));
 
-    // Flags
-    if (n->id_ == myid_) node_str.append("myself,");
+    // Role
     if (n->role_ == kClusterMaster) {
       node_str.append("master - ");
     } else {
       node_str.append("slave " + n->master_id_ + " ");
     }
 
-    // Ping sent, pong received, config epoch, link status
-    auto now = Util::GetTimeStampMS();
-    node_str.append(std::to_string(now - 1) + " " + std::to_string(now) + " " + std::to_string(version_) + " " +
-                    "connected");
-
     // Slots
-    if (n->slots_info_.size() > 0) n->slots_info_.pop_back();  // Trim space
     if (n->role_ == kClusterMaster && n->slots_info_.size() > 0) {
       node_str.append(" " + n->slots_info_);
     }
-    n->slots_info_.clear();  // Reset
-
-    nodes_desc.append(node_str + "\n");
+    nodes_info.append(node_str + "\n");
   }
-  return nodes_desc;
+  return nodes_info;
+}
+
+Status Cluster::DumpClusterNodes(const std::string &file) {
+  // Parse and validate the cluster nodes string before dumping into file
+  std::string tmp_path = file + ".tmp";
+  remove(tmp_path.data());
+  std::ofstream output_file(tmp_path, std::ios::out);
+  output_file << fmt::format("version {}\n", version_);
+  output_file << fmt::format("id {}\n", myid_);
+  output_file << GenNodesInfo();
+  output_file.close();
+  if (rename(tmp_path.data(), file.data()) < 0) {
+    return {Status::NotOK, fmt::format("rename file encounter error: {}", strerror(errno))};
+  }
+  return Status::OK();
+}
+
+Status Cluster::LoadClusterNodes(const std::string &file_path) {
+  if (rocksdb::Env::Default()->FileExists(file_path).IsNotFound()) {
+    LOG(INFO) << fmt::format("The cluster nodes file {} is not found. Use CLUSTERX subcommands to specify it.",
+                             file_path);
+    return Status::OK();
+  }
+
+  std::ifstream file;
+  file.open(file_path);
+  if (!file.is_open()) {
+    return {Status::NotOK, fmt::format("error opening the file '{}': {}", file_path, strerror(errno))};
+  }
+
+  int64_t version = -1;
+  std::string id, nodesInfo;
+  std::string line;
+  while (!file.eof()) {
+    std::getline(file, line);
+
+    auto parsed = ParseConfigLine(line);
+    if (!parsed) return parsed.ToStatus().Prefixed("malformed line");
+    if (parsed->first.empty() || parsed->second.empty()) continue;
+
+    auto key = parsed->first;
+    if (key == "version") {
+      auto parse_result = ParseInt<int64_t>(parsed->second, 10);
+      if (!parse_result) {
+        return {Status::NotOK, errInvalidClusterVersion};
+      }
+      version = *parse_result;
+    } else if (key == "id") {
+      id = parsed->second;
+      if (id.length() != kClusterNodeIdLen) {
+        return {Status::NotOK, errInvalidNodeID};
+      }
+    } else if (key == "node") {
+      nodesInfo.append(parsed->second + "\n");
+    } else {
+      return {Status::NotOK, fmt::format("unknown key: {}", key)};
+    }
+  }
+  return SetClusterNodes(nodesInfo, version, false);
 }
 
 Status Cluster::ParseClusterNodes(const std::string &nodes_str, ClusterNodes *nodes,
                                   std::unordered_map<int, std::string> *slots_nodes) {
   std::vector<std::string> nodes_info = Util::Split(nodes_str, "\n");
   if (nodes_info.size() == 0) {
-    return Status(Status::ClusterInvalidInfo, errInvalidClusterNodeInfo);
+    return {Status::ClusterInvalidInfo, errInvalidClusterNodeInfo};
   }
+
   nodes->clear();
 
   // Parse all nodes
   for (const auto &node_str : nodes_info) {
     std::vector<std::string> fields = Util::Split(node_str, " ");
     if (fields.size() < 5) {
-      return Status(Status::ClusterInvalidInfo, errInvalidClusterNodeInfo);
+      return {Status::ClusterInvalidInfo, errInvalidClusterNodeInfo};
     }
 
     // 1) node id
     if (fields[0].size() != kClusterNodeIdLen) {
-      return Status(Status::ClusterInvalidInfo, errInvalidNodeID);
+      return {Status::ClusterInvalidInfo, errInvalidNodeID};
     }
+
     std::string id = fields[0];
 
     // 2) host, TODO(@shooterit): check host is valid
@@ -539,8 +675,9 @@ Status Cluster::ParseClusterNodes(const std::string &nodes_str, ClusterNodes *no
     // 3) port
     auto parse_result = ParseInt<uint16_t>(fields[2], 10);
     if (!parse_result) {
-      return Status(Status::ClusterInvalidInfo, "Invalid cluster node port");
+      return {Status::ClusterInvalidInfo, "Invalid cluster node port"};
     }
+
     int port = *parse_result;
 
     // 4) role
@@ -550,20 +687,20 @@ Status Cluster::ParseClusterNodes(const std::string &nodes_str, ClusterNodes *no
     } else if (strcasecmp(fields[3].c_str(), "slave") == 0 || strcasecmp(fields[3].c_str(), "replica") == 0) {
       role = kClusterSlave;
     } else {
-      return Status(Status::ClusterInvalidInfo, "Invalid cluster node role");
+      return {Status::ClusterInvalidInfo, "Invalid cluster node role"};
     }
 
     // 5) master id
     std::string master_id = fields[4];
     if ((role == kClusterMaster && master_id != "-") ||
         (role == kClusterSlave && master_id.size() != kClusterNodeIdLen)) {
-      return Status(Status::ClusterInvalidInfo, errInvalidNodeID);
+      return {Status::ClusterInvalidInfo, errInvalidNodeID};
     }
 
     std::bitset<kClusterSlots> slots;
     if (role == kClusterSlave) {
       if (fields.size() != 5) {
-        return Status(Status::ClusterInvalidInfo, errInvalidClusterNodeInfo);
+        return {Status::ClusterInvalidInfo, errInvalidClusterNodeInfo};
       } else {
         // Create slave node
         (*nodes)[id] = std::make_shared<ClusterNode>(id, host, port, role, master_id, slots);
@@ -574,18 +711,18 @@ Status Cluster::ParseClusterNodes(const std::string &nodes_str, ClusterNodes *no
     // 6) slot info
     auto valid_range = NumericRange<int>{0, kClusterSlots - 1};
     for (unsigned i = 5; i < fields.size(); i++) {
-      int start = 0, stop = 0;
       std::vector<std::string> ranges = Util::Split(fields[i], "-");
       if (ranges.size() == 1) {
-        auto parse_result = ParseInt<int>(ranges[0], valid_range, 10);
-        if (!parse_result) {
-          return Status(Status::ClusterInvalidInfo, errSlotOutOfRange);
+        auto parse_start = ParseInt<int>(ranges[0], valid_range, 10);
+        if (!parse_start) {
+          return {Status::ClusterInvalidInfo, errSlotOutOfRange};
         }
-        start = *parse_result;
+
+        int start = *parse_start;
         slots.set(start, true);
         if (role == kClusterMaster) {
           if (slots_nodes->find(start) != slots_nodes->end()) {
-            return Status(Status::ClusterInvalidInfo, errSlotOverlapped);
+            return {Status::ClusterInvalidInfo, errSlotOverlapped};
           } else {
             (*slots_nodes)[start] = id;
           }
@@ -594,28 +731,30 @@ Status Cluster::ParseClusterNodes(const std::string &nodes_str, ClusterNodes *no
         auto parse_start = ParseInt<int>(ranges[0], valid_range, 10);
         auto parse_stop = ParseInt<int>(ranges[1], valid_range, 10);
         if (!parse_start || !parse_stop || *parse_start >= *parse_stop) {
-          return Status(Status::ClusterInvalidInfo, errSlotOutOfRange);
+          return {Status::ClusterInvalidInfo, errSlotOutOfRange};
         }
-        start = *parse_start;
-        stop = *parse_stop;
+
+        int start = *parse_start;
+        int stop = *parse_stop;
         for (int j = start; j <= stop; j++) {
           slots.set(j, true);
           if (role == kClusterMaster) {
             if (slots_nodes->find(j) != slots_nodes->end()) {
-              return Status(Status::ClusterInvalidInfo, errSlotOverlapped);
+              return {Status::ClusterInvalidInfo, errSlotOverlapped};
             } else {
               (*slots_nodes)[j] = id;
             }
           }
         }
       } else {
-        return Status(Status::ClusterInvalidInfo, errSlotOutOfRange);
+        return {Status::ClusterInvalidInfo, errSlotOutOfRange};
       }
     }
 
     // Create master node
     (*nodes)[id] = std::make_shared<ClusterNode>(id, host, port, role, master_id, slots);
   }
+
   return Status::OK();
 }
 
@@ -632,33 +771,36 @@ Status Cluster::CanExecByMySelf(const Redis::CommandAttributes *attributes, cons
   auto s = Redis::GetKeysFromCommand(attributes->name, static_cast<int>(cmd_tokens.size()), &keys_indexes);
   // No keys
   if (!s.IsOK()) return Status::OK();
+
   if (keys_indexes.size() == 0) return Status::OK();
 
   int slot = -1;
   for (auto i : keys_indexes) {
     if (i >= static_cast<int>(cmd_tokens.size())) break;
+
     int cur_slot = GetSlotNumFromKey(cmd_tokens[i]);
     if (slot == -1) slot = cur_slot;
     if (slot != cur_slot) {
-      return Status(Status::RedisExecErr, "CROSSSLOT Attempted to access keys that don't hash to the same slot");
+      return {Status::RedisExecErr, "CROSSSLOT Attempted to access keys that don't hash to the same slot"};
     }
   }
   if (slot == -1) return Status::OK();
 
   if (slots_nodes_[slot] == nullptr) {
-    return Status(Status::ClusterDown, "CLUSTERDOWN Hash slot not served");
+    return {Status::ClusterDown, "CLUSTERDOWN Hash slot not served"};
   } else if (myself_ && myself_ == slots_nodes_[slot]) {
     // We use central controller to manage the topology of the cluster.
     // Server can't change the topology directly, so we record the migrated slots
     // to move the requests of the migrated slots to the destination node.
-    if (migrated_slots_.count(slot)) {  // I'm not serving the migrated slot
-      return Status(Status::RedisExecErr, "MOVED " + std::to_string(slot) + " " + migrated_slots_[slot]);
+    if (migrated_slots_.count(slot) > 0) {  // I'm not serving the migrated slot
+      return {Status::RedisExecErr, fmt::format("MOVED {} {}", slot, migrated_slots_[slot])};
     }
     // To keep data consistency, slot will be forbidden write while sending the last incremental data.
     // During this phase, the requests of the migrating slot has to be rejected.
     if (attributes->is_write() && IsWriteForbiddenSlot(slot)) {
-      return Status(Status::RedisExecErr, "Can't write to slot being migrated which is in write forbidden phase");
+      return {Status::RedisExecErr, "TRYAGAIN Can't write to slot being migrated which is in write forbidden phase"};
     }
+
     return Status::OK();  // I'm serving this slot
   } else if (myself_ && myself_->importing_slot_ == slot && conn->IsImporting()) {
     // While data migrating, the topology of the destination node has not been changed.
@@ -671,11 +813,11 @@ Status Cluster::CanExecByMySelf(const Redis::CommandAttributes *attributes, cons
     // the destination server. Before the central controller change the topology, the destination
     // server should record the imported slots to accept new data of the imported slots.
     return Status::OK();  // I'm serving the imported slot
-  } else if (myself_ && myself_->role_ == kClusterSlave && attributes->is_write() == false &&
+  } else if (myself_ && myself_->role_ == kClusterSlave && !attributes->is_write() &&
              nodes_.find(myself_->master_id_) != nodes_.end() && nodes_[myself_->master_id_] == slots_nodes_[slot]) {
-    return Status::OK();  // My mater is serving this slot
+    return Status::OK();  // My master is serving this slot
   } else {
-    std::string ip_port = slots_nodes_[slot]->host_ + ":" + std::to_string(slots_nodes_[slot]->port_);
-    return Status(Status::RedisExecErr, "MOVED " + std::to_string(slot) + " " + ip_port);
+    return {Status::RedisExecErr,
+            fmt::format("MOVED {} {}:{}", slot, slots_nodes_[slot]->host_, slots_nodes_[slot]->port_)};
   }
 }
