@@ -20,17 +20,16 @@
 
 #include "config.h"
 
-#include <fcntl.h>
+#include <fmt/format.h>
 #include <rocksdb/env.h>
 #include <strings.h>
 
-#include <algorithm>
-#include <cctype>
 #include <cstring>
 #include <fstream>
 #include <iostream>
+#include <iterator>
 #include <limits>
-#include <sstream>
+#include <string>
 #include <utility>
 #include <vector>
 
@@ -38,38 +37,40 @@
 #include "config_util.h"
 #include "parse_util.h"
 #include "server/server.h"
-#include "server/tls_util.h"
 #include "status.h"
-#include "util.h"
 
 const char *kDefaultNamespace = "__namespace";
-
-const char *errNotEnableBlobDB = "Must set rocksdb.enable_blob_files to yes first.";
-
-const char *errNotSetLevelCompactionDynamicLevelBytes =
-    "Must set rocksdb.level_compaction_dynamic_level_bytes yes first.";
-
 const char *kDefaultBindAddress = "127.0.0.1";
 
-configEnum compression_type_enum[] = {
+const char *errBlobDbNotEnabled = "Must set rocksdb.enable_blob_files to yes first.";
+const char *errLevelCompactionDynamicLevelBytesNotSet =
+    "Must set rocksdb.level_compaction_dynamic_level_bytes yes first.";
+
+configEnum compression_types[] = {
     {"no", rocksdb::CompressionType::kNoCompression},     {"snappy", rocksdb::CompressionType::kSnappyCompression},
     {"lz4", rocksdb::CompressionType::kLZ4Compression},   {"zstd", rocksdb::CompressionType::kZSTD},
     {"zlib", rocksdb::CompressionType::kZlibCompression}, {nullptr, 0}};
 
-configEnum supervised_mode_enum[] = {{"no", kSupervisedNone},
-                                     {"auto", kSupervisedAutoDetect},
-                                     {"upstart", kSupervisedUpStart},
-                                     {"systemd", kSupervisedSystemd},
-                                     {nullptr, 0}};
+configEnum supervised_modes[] = {{"no", kSupervisedNone},
+                                 {"auto", kSupervisedAutoDetect},
+                                 {"upstart", kSupervisedUpStart},
+                                 {"systemd", kSupervisedSystemd},
+                                 {nullptr, 0}};
+
+configEnum log_levels[] = {{"info", google::INFO},
+                           {"warning", google::WARNING},
+                           {"error", google::ERROR},
+                           {"fatal", google::FATAL},
+                           {nullptr, 0}};
 
 std::string trimRocksDBPrefix(std::string s) {
-  if (strncasecmp(s.data(), "rocksdb.", 8)) return s;
+  if (strncasecmp(s.data(), "rocksdb.", 8) != 0) return s;
   return s.substr(8, s.size() - 8);
 }
 
 int configEnumGetValue(configEnum *ce, const char *name) {
   while (ce->name != nullptr) {
-    if (!strcasecmp(ce->name, name)) return ce->val;
+    if (strcasecmp(ce->name, name) == 0) return ce->val;
     ce++;
   }
   return INT_MIN;
@@ -95,9 +96,9 @@ Config::Config() {
   FieldWrapper fields[] = {
       {"daemonize", true, new YesNoField(&daemonize, false)},
       {"bind", true, new StringField(&binds_, "")},
-      {"port", true, new IntField(&port, kDefaultPort, 1, PORT_LIMIT)},
+      {"port", true, new UInt32Field(&port, kDefaultPort, 1, PORT_LIMIT)},
 #ifdef ENABLE_OPENSSL
-      {"tls-port", true, new IntField(&tls_port, 0, 0, PORT_LIMIT)},
+      {"tls-port", true, new UInt32Field(&tls_port, 0, 0, PORT_LIMIT)},
       {"tls-cert-file", false, new StringField(&tls_cert_file, "")},
       {"tls-key-file", false, new StringField(&tls_key_file, "")},
       {"tls-key-file-pass", false, new StringField(&tls_key_file_pass, "")},
@@ -129,12 +130,13 @@ Config::Config() {
       {"dir", true, new StringField(&dir, "/tmp/kvrocks")},
       {"backup-dir", false, new StringField(&backup_dir, "")},
       {"log-dir", true, new StringField(&log_dir, "")},
+      {"log-level", true, new EnumField(&log_level, log_levels, google::INFO)},
       {"pidfile", true, new StringField(&pidfile, "")},
       {"max-io-mb", false, new IntField(&max_io_mb, 500, 0, INT_MAX)},
       {"max-bitmap-to-string-mb", false, new IntField(&max_bitmap_to_string_mb, 16, 0, INT_MAX)},
       {"max-db-size", false, new IntField(&max_db_size, 0, 0, INT_MAX)},
       {"max-replication-mb", false, new IntField(&max_replication_mb, 0, 0, INT_MAX)},
-      {"supervised", true, new EnumField(&supervised_mode, supervised_mode_enum, kSupervisedNone)},
+      {"supervised", true, new EnumField(&supervised_mode, supervised_modes, kSupervisedNone)},
       {"slave-serve-stale-data", false, new YesNoField(&slave_serve_stale_data, true)},
       {"slave-empty-db-before-fullsync", false, new YesNoField(&slave_empty_db_before_fullsync, false)},
       {"slave-priority", false, new IntField(&slave_priority, 100, 0, INT_MAX)},
@@ -157,9 +159,12 @@ Config::Config() {
       {"migrate-sequence-gap", false, new IntField(&sequence_gap, 10000, 1, INT_MAX)},
       {"unixsocket", true, new StringField(&unixsocket, "")},
       {"unixsocketperm", true, new OctalField(&unixsocketperm, 0777, 1, INT_MAX)},
+      {"log-retention-days", false, new IntField(&log_retention_days, -1, -1, INT_MAX)},
+      {"persist-cluster-nodes-enabled", false, new YesNoField(&persist_cluster_nodes_enabled, true)},
 
       /* rocksdb options */
-      {"rocksdb.compression", false, new EnumField(&RocksDB.compression, compression_type_enum, 0)},
+      {"rocksdb.compression", false,
+       new EnumField(&RocksDB.compression, compression_types, rocksdb::CompressionType::kNoCompression)},
       {"rocksdb.block_size", true, new IntField(&RocksDB.block_size, 4096, 0, INT_MAX)},
       {"rocksdb.max_open_files", false, new IntField(&RocksDB.max_open_files, 4096, -1, INT_MAX)},
       {"rocksdb.write_buffer_size", false, new IntField(&RocksDB.write_buffer_size, 64, 0, 4096)},
@@ -208,6 +213,9 @@ Config::Config() {
       {"rocksdb.write_options.low_pri", true, new YesNoField(&RocksDB.write_options.low_pri, false)},
       {"rocksdb.write_options.memtable_insert_hint_per_batch", true,
        new YesNoField(&RocksDB.write_options.memtable_insert_hint_per_batch, false)},
+
+      /* rocksdb read options */
+      {"rocksdb.read_options.async_io", false, new YesNoField(&RocksDB.read_options.async_io, false)},
   };
   for (auto &wrapper : fields) {
     auto &field = wrapper.field;
@@ -225,17 +233,17 @@ void Config::initFieldValidator() {
       {"requirepass",
        [this](const std::string &k, const std::string &v) -> Status {
          if (v.empty() && !tokens.empty()) {
-           return Status(Status::NotOK, "requirepass empty not allowed while the namespace exists");
+           return {Status::NotOK, "requirepass empty not allowed while the namespace exists"};
          }
          if (tokens.find(v) != tokens.end()) {
-           return Status(Status::NotOK, "requirepass is duplicated with namespace tokens");
+           return {Status::NotOK, "requirepass is duplicated with namespace tokens"};
          }
          return Status::OK();
        }},
       {"masterauth",
        [this](const std::string &k, const std::string &v) -> Status {
          if (tokens.find(v) != tokens.end()) {
-           return Status(Status::NotOK, "masterauth is duplicated with namespace tokens");
+           return {Status::NotOK, "masterauth is duplicated with namespace tokens"};
          }
          return Status::OK();
        }},
@@ -258,14 +266,11 @@ void Config::initFieldValidator() {
          }
          std::vector<std::string> args = Util::Split(v, "-");
          if (args.size() != 2) {
-           return Status(Status::NotOK, "invalid range format, the range should be between 0 and 24");
+           return {Status::NotOK, "invalid range format, the range should be between 0 and 24"};
          }
-         int64_t start, stop;
-         Status s = Util::DecimalStringToNum(args[0], &start, 0, 24);
-         if (!s.IsOK()) return s;
-         s = Util::DecimalStringToNum(args[1], &stop, 0, 24);
-         if (!s.IsOK()) return s;
-         if (start > stop) return Status(Status::NotOK, "invalid range format, start should be smaller than stop");
+         auto start = GET_OR_RET(ParseInt<int>(args[0], {0, 24}, 10)),
+              stop = GET_OR_RET(ParseInt<int>(args[1], {0, 24}, 10));
+         if (start > stop) return {Status::NotOK, "invalid range format, start should be smaller than stop"};
          compaction_checker_range.Start = start;
          compaction_checker_range.Stop = stop;
          return Status::OK();
@@ -276,17 +281,17 @@ void Config::initFieldValidator() {
          for (auto &p : all_args) {
            std::vector<std::string> args = Util::Split(p, " \t");
            if (args.size() != 2) {
-             return Status(Status::NotOK, "Invalid rename-command format");
+             return {Status::NotOK, "Invalid rename-command format"};
            }
            auto commands = Redis::GetCommands();
            auto cmd_iter = commands->find(Util::ToLower(args[0]));
            if (cmd_iter == commands->end()) {
-             return Status(Status::NotOK, "No such command in rename-command");
+             return {Status::NotOK, "No such command in rename-command"};
            }
            if (args[1] != "\"\"") {
              auto new_command_name = Util::ToLower(args[1]);
              if (commands->find(new_command_name) != commands->end()) {
-               return Status(Status::NotOK, "Target command name already exists");
+               return {Status::NotOK, "Target command name already exists"};
              }
              (*commands)[new_command_name] = cmd_iter->second;
            }
@@ -366,7 +371,6 @@ void Config::initFieldCallback() {
        }},
       {"bind",
        [this](Server *srv, const std::string &k, const std::string &v) -> Status {
-         trimRocksDBPrefix(k);
          std::vector<std::string> args = Util::Split(v, " \t");
          binds = std::move(args);
          return Status::OK();
@@ -383,12 +387,12 @@ void Config::initFieldCallback() {
            return Status::OK();
          }
          std::vector<std::string> args = Util::Split(v, " \t");
-         if (args.size() != 2) return Status(Status::NotOK, "wrong number of arguments");
+         if (args.size() != 2) return {Status::NotOK, "wrong number of arguments"};
          if (args[0] != "no" && args[1] != "one") {
            master_host = args[0];
            auto parse_result = ParseInt<int>(args[1].c_str(), NumericRange<int>{1, PORT_LIMIT - 1}, 10);
            if (!parse_result) {
-             return Status(Status::NotOK, "should be between 0 and 65535");
+             return {Status::NotOK, "should be between 0 and 65535"};
            }
            master_port = *parse_result;
          }
@@ -406,7 +410,7 @@ void Config::initFieldCallback() {
              return Status::OK();
            }
            if (!Redis::IsCommandExists(cmd)) {
-             return Status(Status::NotOK, cmd + " is not Kvrocks supported command");
+             return {Status::NotOK, cmd + " is not Kvrocks supported command"};
            }
            // profiling_sample_commands use command's original name, regardless of rename-command directive
            profiling_sample_commands.insert(cmd);
@@ -428,7 +432,7 @@ void Config::initFieldCallback() {
       {"max-io-mb",
        [this](Server *srv, const std::string &k, const std::string &v) -> Status {
          if (!srv) return Status::OK();
-         srv->storage_->SetIORateLimit(static_cast<uint64_t>(max_io_mb));
+         srv->storage_->SetIORateLimit(max_io_mb);
          return Status::OK();
        }},
       {"profiling-sample-record-max-len",
@@ -455,6 +459,31 @@ void Config::initFieldCallback() {
          if (cluster_enabled) srv->slot_migrate_->SetSequenceGapSize(sequence_gap);
          return Status::OK();
        }},
+      {"log-retention-days",
+       [this](Server *srv, const std::string &k, const std::string &v) -> Status {
+         if (!srv) return Status::OK();
+         if (Util::ToLower(log_dir) == "stdout") {
+           return {Status::NotOK, "can't set the 'log-retention-days' when the log dir is stdout"};
+         }
+
+         if (log_retention_days != -1) {
+           google::EnableLogCleaner(log_retention_days);
+         } else {
+           google::DisableLogCleaner();
+         }
+         return Status::OK();
+       }},
+      {"persist-cluster-nodes-enabled",
+       [this](Server *srv, const std::string &k, const std::string &v) -> Status {
+         if (!srv || !cluster_enabled) return Status::OK();
+         auto nodes_file_path = NodesFilePath();
+         if (v == "yes") {
+           return srv->cluster_->DumpClusterNodes(nodes_file_path);
+         }
+         // Remove the cluster nodes file to avoid stale cluster nodes info
+         remove(nodes_file_path.data());
+         return Status::OK();
+       }},
       {"rocksdb.target_file_size_base",
        [this](Server *srv, const std::string &k, const std::string &v) -> Status {
          if (!srv) return Status::OK();
@@ -488,7 +517,7 @@ void Config::initFieldCallback() {
        [this](Server *srv, const std::string &k, const std::string &v) -> Status {
          if (!srv) return Status::OK();
          if (!RocksDB.enable_blob_files) {
-           return Status(Status::NotOK, errNotEnableBlobDB);
+           return {Status::NotOK, errBlobDbNotEnabled};
          }
          return srv->storage_->SetColumnFamilyOption(trimRocksDBPrefix(k), v);
        }},
@@ -496,7 +525,7 @@ void Config::initFieldCallback() {
        [this](Server *srv, const std::string &k, const std::string &v) -> Status {
          if (!srv) return Status::OK();
          if (!RocksDB.enable_blob_files) {
-           return Status(Status::NotOK, errNotEnableBlobDB);
+           return {Status::NotOK, errBlobDbNotEnabled};
          }
          return srv->storage_->SetColumnFamilyOption(trimRocksDBPrefix(k), std::to_string(RocksDB.blob_file_size));
        }},
@@ -504,7 +533,7 @@ void Config::initFieldCallback() {
        [this](Server *srv, const std::string &k, const std::string &v) -> Status {
          if (!srv) return Status::OK();
          if (!RocksDB.enable_blob_files) {
-           return Status(Status::NotOK, errNotEnableBlobDB);
+           return {Status::NotOK, errBlobDbNotEnabled};
          }
          std::string enable_blob_garbage_collection = v == "yes" ? "true" : "false";
          return srv->storage_->SetColumnFamilyOption(trimRocksDBPrefix(k), enable_blob_garbage_collection);
@@ -513,16 +542,16 @@ void Config::initFieldCallback() {
        [this](Server *srv, const std::string &k, const std::string &v) -> Status {
          if (!srv) return Status::OK();
          if (!RocksDB.enable_blob_files) {
-           return Status(Status::NotOK, errNotEnableBlobDB);
+           return {Status::NotOK, errBlobDbNotEnabled};
          }
-         int val;
+         int val = 0;
          auto parse_result = ParseInt<int>(v, 10);
          if (!parse_result) {
-           return Status(Status::NotOK, "Illegal blob_garbage_collection_age_cutoff value.");
+           return {Status::NotOK, "Illegal blob_garbage_collection_age_cutoff value."};
          }
          val = *parse_result;
          if (val < 0 || val > 100) {
-           return Status(Status::NotOK, "blob_garbage_collection_age_cutoff must >= 0 and <= 100.");
+           return {Status::NotOK, "blob_garbage_collection_age_cutoff must >= 0 and <= 100."};
          }
 
          double cutoff = val / 100.0;
@@ -538,7 +567,7 @@ void Config::initFieldCallback() {
        [this](Server *srv, const std::string &k, const std::string &v) -> Status {
          if (!srv) return Status::OK();
          if (!RocksDB.level_compaction_dynamic_level_bytes) {
-           return Status(Status::NotOK, errNotSetLevelCompactionDynamicLevelBytes);
+           return {Status::NotOK, errLevelCompactionDynamicLevelBytesNotSet};
          }
          return srv->storage_->SetColumnFamilyOption(trimRocksDBPrefix(k),
                                                      std::to_string(RocksDB.max_bytes_for_level_base));
@@ -547,7 +576,7 @@ void Config::initFieldCallback() {
        [this](Server *srv, const std::string &k, const std::string &v) -> Status {
          if (!srv) return Status::OK();
          if (!RocksDB.level_compaction_dynamic_level_bytes) {
-           return Status(Status::NotOK, errNotSetLevelCompactionDynamicLevelBytes);
+           return {Status::NotOK, errLevelCompactionDynamicLevelBytesNotSet};
          }
          return srv->storage_->SetColumnFamilyOption(trimRocksDBPrefix(k), v);
        }},
@@ -586,12 +615,17 @@ void Config::initFieldCallback() {
   }
 }
 
-void Config::SetMaster(const std::string &host, int port) {
+std::string Config::NodesFilePath() { return dir + "/nodes.conf"; }
+
+void Config::SetMaster(const std::string &host, uint32_t port) {
   master_host = host;
   master_port = port;
   auto iter = fields_.find("slaveof");
   if (iter != fields_.end()) {
-    iter->second->Set(master_host + " " + std::to_string(master_port));
+    auto s = iter->second->Set(master_host + " " + std::to_string(master_port));
+    if (!s.IsOK()) {
+      LOG(ERROR) << "Failed to set the value of 'slaveof' setting: " << s.Msg();
+    }
   }
 }
 
@@ -600,7 +634,10 @@ void Config::ClearMaster() {
   master_port = 0;
   auto iter = fields_.find("slaveof");
   if (iter != fields_.end()) {
-    iter->second->Set("no one");
+    auto s = iter->second->Set("no one");
+    if (!s.IsOK()) {
+      LOG(ERROR) << "Failed to clear the value of 'slaveof' setting: " << s.Msg();
+    }
   }
 }
 
@@ -608,7 +645,7 @@ Status Config::parseConfigFromPair(const std::pair<std::string, std::string> &in
   std::string field_key = Util::ToLower(input.first);
   const char ns_str[] = "namespace.";
   size_t ns_str_size = sizeof(ns_str) - 1;
-  if (!strncasecmp(input.first.data(), ns_str, ns_str_size)) {
+  if (strncasecmp(input.first.data(), ns_str, ns_str_size) == 0) {
     // namespace should keep key case-sensitive
     field_key = input.first;
     tokens[input.second] = input.first.substr(ns_str_size);
@@ -618,14 +655,14 @@ Status Config::parseConfigFromPair(const std::pair<std::string, std::string> &in
     auto &field = iter->second;
     field->line_number = line_number;
     auto s = field->Set(input.second);
-    if (!s.IsOK()) return s;
+    if (!s.IsOK()) return s.Prefixed(fmt::format("failed to set value of field '{}'", field_key));
   }
   return Status::OK();
 }
 
 Status Config::parseConfigFromString(const std::string &input, int line_number) {
   auto parsed = ParseConfigLine(input);
-  if (!parsed) return parsed.ToStatus();
+  if (!parsed) return parsed.ToStatus().Prefixed("malformed line");
 
   auto kv = std::move(*parsed);
 
@@ -636,21 +673,21 @@ Status Config::parseConfigFromString(const std::string &input, int line_number)
 
 Status Config::finish() {
   if (requirepass.empty() && !tokens.empty()) {
-    return Status(Status::NotOK, "requirepass empty wasn't allowed while the namespace exists");
+    return {Status::NotOK, "requirepass empty wasn't allowed while the namespace exists"};
   }
   if ((cluster_enabled) && !tokens.empty()) {
-    return Status(Status::NotOK, "enabled cluster mode wasn't allowed while the namespace exists");
+    return {Status::NotOK, "enabled cluster mode wasn't allowed while the namespace exists"};
   }
   if (unixsocket.empty() && binds.size() == 0) {
     binds.emplace_back(kDefaultBindAddress);
   }
   if (cluster_enabled && binds.size() == 0) {
-    return Status(Status::NotOK,
-                  "node is in cluster mode, but TCP listen address "
-                  "wasn't specified via configuration file");
+    return {Status::NotOK,
+            "node is in cluster mode, but TCP listen address "
+            "wasn't specified via configuration file"};
   }
   if (master_port != 0 && binds.size() == 0) {
-    return Status(Status::NotOK, "replication doesn't supports unix socket");
+    return {Status::NotOK, "replication doesn't support unix socket"};
   }
   if (db_dir.empty()) db_dir = dir + "/db";
   if (backup_dir.empty()) backup_dir = dir + "/backup";
@@ -659,7 +696,7 @@ Status Config::finish() {
   std::vector<std::string> createDirs = {dir};
   for (const auto &name : createDirs) {
     auto s = rocksdb::Env::Default()->CreateDirIfMissing(name);
-    if (!s.ok()) return Status(Status::NotOK, s.ToString());
+    if (!s.ok()) return {Status::NotOK, s.ToString()};
   }
   return Status::OK();
 }
@@ -673,7 +710,10 @@ Status Config::Load(const CLIOptions &opts) {
     } else {
       path_ = opts.conf_file;
       file.open(path_);
-      if (!file.is_open()) return Status(Status::NotOK, strerror(errno));
+      if (!file.is_open()) {
+        return {Status::NotOK, fmt::format("failed to open file '{}': {}", path_, strerror(errno))};
+      }
+
       in = &file;
     }
 
@@ -681,21 +721,19 @@ Status Config::Load(const CLIOptions &opts) {
     int line_num = 1;
     while (!in->eof()) {
       std::getline(*in, line);
-      if (auto s = parseConfigFromString(line, line_num); !s) {
-        return Status(Status::NotOK, "at line: #L" + std::to_string(line_num) + ", err: " + s.Msg());
+      if (auto s = parseConfigFromString(line, line_num); !s.IsOK()) {
+        return s.Prefixed(fmt::format("at line #L{}", line_num));
       }
+
       line_num++;
     }
   } else {
-    std::cout << "Warn: no config file specified, using the default config. "
-                 "In order to specify a config file use kvrocks -c /path/to/kvrocks.conf"
-              << std::endl;
+    std::cout << "WARNING: No config file specified, using the default configuration. "
+              << "In order to specify a config file use 'kvrocks -c /path/to/kvrocks.conf'" << std::endl;
   }
 
   for (const auto &opt : opts.cli_options) {
-    if (Status s = parseConfigFromPair(opt, -1); !s) {
-      return Status(Status::NotOK, "CLI config option error: " + s.Msg());
-    }
+    GET_OR_RET(parseConfigFromPair(opt, -1).Prefixed("CLI config option error"));
   }
 
   for (const auto &iter : fields_) {
@@ -704,8 +742,7 @@ Status Config::Load(const CLIOptions &opts) {
     if (iter.second->line_number != 0 && iter.second->validate) {
       auto s = iter.second->validate(iter.first, iter.second->ToString());
       if (!s.IsOK()) {
-        return Status(Status::NotOK, "at line: #L" + std::to_string(iter.second->line_number) + ", " + iter.first +
-                                         " is invalid: " + s.Msg());
+        return s.Prefixed(fmt::format("at line #L{}: {} is invalid", iter.second->line_number, iter.first));
       }
     }
   }
@@ -714,14 +751,14 @@ Status Config::Load(const CLIOptions &opts) {
     if (iter.second->callback) {
       auto s = iter.second->callback(nullptr, iter.first, iter.second->ToString());
       if (!s.IsOK()) {
-        return Status(Status::NotOK, s.Msg() + " in key '" + iter.first + "'");
+        return s.Prefixed(fmt::format("while changing key '{}'", iter.first));
       }
     }
   }
   return finish();
 }
 
-void Config::Get(std::string key, std::vector<std::string> *values) {
+void Config::Get(const std::string &key, std::vector<std::string> *values) {
   values->clear();
   for (const auto &iter : fields_) {
     if (key == "*" || Util::ToLower(key) == iter.first) {
@@ -742,7 +779,7 @@ Status Config::Set(Server *svr, std::string key, const std::string &value) {
   key = Util::ToLower(key);
   auto iter = fields_.find(key);
   if (iter == fields_.end() || iter->second->readonly) {
-    return Status(Status::NotOK, "Unsupported CONFIG parameter: " + key);
+    return {Status::NotOK, "Unsupported CONFIG parameter: " + key};
   }
   auto &field = iter->second;
   if (field->validate) {
@@ -759,8 +796,9 @@ Status Config::Set(Server *svr, std::string key, const std::string &value) {
 
 Status Config::Rewrite() {
   if (path_.empty()) {
-    return Status(Status::NotOK, "the server is running without a config file");
+    return {Status::NotOK, "the server is running without a config file"};
   }
+
   std::vector<std::string> lines;
   std::map<std::string, std::string> new_config;
   for (const auto &iter : fields_) {
@@ -803,21 +841,21 @@ Status Config::Rewrite() {
   }
   file.close();
 
-  std::ostringstream string_stream;
+  std::string out_buf;
   for (const auto &line : lines) {
-    string_stream << line << "\n";
+    fmt::format_to(std::back_inserter(out_buf), "{}\n", line);
   }
   for (const auto &remain : new_config) {
     if (remain.second.empty()) continue;
-    string_stream << remain.first << " " << remain.second << "\n";
+    fmt::format_to(std::back_inserter(out_buf), "{} {}\n", remain.first, remain.second);
   }
   std::string tmp_path = path_ + ".tmp";
   remove(tmp_path.data());
   std::ofstream output_file(tmp_path, std::ios::out);
-  output_file.write(string_stream.str().c_str(), string_stream.str().size());
+  output_file << out_buf;
   output_file.close();
   if (rename(tmp_path.data(), path_.data()) < 0) {
-    return Status(Status::NotOK, std::string("rename file encounter error: ") + strerror(errno));
+    return {Status::NotOK, fmt::format("rename file encounter error: {}", strerror(errno))};
   }
   return Status::OK();
 }
@@ -830,19 +868,19 @@ Status Config::GetNamespace(const std::string &ns, std::string *token) {
       return Status::OK();
     }
   }
-  return Status(Status::NotFound);
+  return {Status::NotFound};
 }
 
 Status Config::SetNamespace(const std::string &ns, const std::string &token) {
   if (ns == kDefaultNamespace) {
-    return Status(Status::NotOK, "forbidden to update the default namespace");
+    return {Status::NotOK, "forbidden to update the default namespace"};
   }
   if (tokens.find(token) != tokens.end()) {
-    return Status(Status::NotOK, "the token has already exists");
+    return {Status::NotOK, "the token has already exists"};
   }
 
   if (token == requirepass || token == masterauth) {
-    return Status(Status::NotOK, "the token is duplicated with requirepass or masterauth");
+    return {Status::NotOK, "the token is duplicated with requirepass or masterauth"};
   }
 
   for (const auto &iter : tokens) {
@@ -858,32 +896,32 @@ Status Config::SetNamespace(const std::string &ns, const std::string &token) {
       return s;
     }
   }
-  return Status(Status::NotOK, "the namespace was not found");
+  return {Status::NotOK, "the namespace was not found"};
 }
 
 Status Config::AddNamespace(const std::string &ns, const std::string &token) {
   if (requirepass.empty()) {
-    return Status(Status::NotOK, "forbidden to add namespace when requirepass was empty");
+    return {Status::NotOK, "forbidden to add namespace when requirepass was empty"};
   }
   if (cluster_enabled) {
-    return Status(Status::NotOK, "forbidden to add namespace when cluster mode was enabled");
+    return {Status::NotOK, "forbidden to add namespace when cluster mode was enabled"};
   }
   if (ns == kDefaultNamespace) {
-    return Status(Status::NotOK, "forbidden to add the default namespace");
+    return {Status::NotOK, "forbidden to add the default namespace"};
   }
   auto s = isNamespaceLegal(ns);
   if (!s.IsOK()) return s;
   if (tokens.find(token) != tokens.end()) {
-    return Status(Status::NotOK, "the token has already exists");
+    return {Status::NotOK, "the token has already exists"};
   }
 
   if (token == requirepass || token == masterauth) {
-    return Status(Status::NotOK, "the token is duplicated with requirepass or masterauth");
+    return {Status::NotOK, "the token is duplicated with requirepass or masterauth"};
   }
 
   for (const auto &iter : tokens) {
     if (iter.second == ns) {
-      return Status(Status::NotOK, "the namespace has already exists");
+      return {Status::NotOK, "the namespace has already exists"};
     }
   }
   tokens[token] = ns;
@@ -897,7 +935,7 @@ Status Config::AddNamespace(const std::string &ns, const std::string &token) {
 
 Status Config::DelNamespace(const std::string &ns) {
   if (ns == kDefaultNamespace) {
-    return Status(Status::NotOK, "forbidden to delete the default namespace");
+    return {Status::NotOK, "forbidden to delete the default namespace"};
   }
   for (const auto &iter : tokens) {
     if (iter.second == ns) {
@@ -909,16 +947,16 @@ Status Config::DelNamespace(const std::string &ns) {
       return s;
     }
   }
-  return Status(Status::NotOK, "the namespace was not found");
+  return {Status::NotOK, "the namespace was not found"};
 }
 
 Status Config::isNamespaceLegal(const std::string &ns) {
   if (ns.size() > UINT8_MAX) {
-    return Status(Status::NotOK, std::string("size exceed limit ") + std::to_string(UINT8_MAX));
+    return {Status::NotOK, fmt::format("size exceed limit {}", UINT8_MAX)};
   }
   char last_char = ns.back();
   if (last_char == std::numeric_limits<char>::max()) {
-    return Status(Status::NotOK, std::string("namespace contain illegal letter"));
+    return {Status::NotOK, "namespace contain illegal letter"};
   }
   return Status::OK();
 }
