@@ -27,7 +27,7 @@
 #include <iomanip>
 #include <ostream>
 #ifdef __linux__
-#define _XOPEN_SOURCE 700
+#define _XOPEN_SOURCE 700  // NOLINT
 #else
 #define _XOPEN_SOURCE
 #endif
@@ -40,10 +40,11 @@
 
 #include "config.h"
 #include "fd_util.h"
+#include "io_util.h"
 #include "scope_exit.h"
 #include "server/server.h"
 #include "storage/storage.h"
-#include "util.h"
+#include "string_util.h"
 #include "version.h"
 
 namespace google {
@@ -80,7 +81,7 @@ extern "C" void segvHandler(int sig, siginfo_t *info, void *secret) {
   for (int i = 1; i < trace_size; ++i) {
     char func_info[1024] = {};
     if (google::Symbolize(trace[i], func_info, sizeof(func_info) - 1)) {
-      LOG(ERROR) << std::left << std::setw(max_msg_len) << messages[i] << "  " << func_info;
+      LOG(ERROR) << std::left << std::setw(static_cast<int>(max_msg_len)) << messages[i] << "  " << func_info;
     } else {
       LOG(ERROR) << messages[i];
     }
@@ -165,7 +166,7 @@ static CLIOptions parseCommandLineOptions(int argc, char **argv) {
 }
 
 static void initGoogleLog(const Config *config) {
-  FLAGS_minloglevel = config->loglevel;
+  FLAGS_minloglevel = config->log_level;
   FLAGS_max_log_size = 100;
   FLAGS_logbufsecs = 0;
 
@@ -176,7 +177,10 @@ static void initGoogleLog(const Config *config) {
     FLAGS_stderrthreshold = google::ERROR;
     FLAGS_logtostdout = true;
   } else {
-    FLAGS_log_dir = config->log_dir;
+    FLAGS_log_dir = config->log_dir + "/";
+    if (config->log_retention_days != -1) {
+      google::EnableLogCleaner(config->log_retention_days);
+    }
   }
 }
 
@@ -261,7 +265,10 @@ static Status createPidFile(const std::string &path) {
     return Status(Status::NotOK, strerror(errno));
   }
   std::string pid_str = std::to_string(getpid());
-  Util::Write(*fd, pid_str);
+  auto s = Util::Write(*fd, pid_str);
+  if (!s.IsOK()) {
+    return s.Prefixed("failed to write to PID-file");
+  }
   return Status::OK();
 }
 
@@ -299,20 +306,20 @@ int main(int argc, char *argv[]) {
   Config config;
   Status s = config.Load(opts);
   if (!s.IsOK()) {
-    std::cout << "Failed to load config, err: " << s.Msg() << std::endl;
+    std::cout << "Failed to load config. Error: " << s.Msg() << std::endl;
     return 1;
   }
+
   initGoogleLog(&config);
   printVersion(LOG(INFO));
   // Tricky: We don't expect that different instances running on the same port,
   // but the server use REUSE_PORT to support the multi listeners. So we connect
   // the listen port to check if the port has already listened or not.
   if (!config.binds.empty()) {
-    int ports[] = {config.port, config.tls_port, 0};
-    for (int *port = ports; *port; ++port) {
+    uint32_t ports[] = {config.port, config.tls_port, 0};
+    for (uint32_t *port = ports; *port; ++port) {
       if (Util::IsPortInUse(*port)) {
-        LOG(ERROR) << "Could not create server TCP since the specified port[" << *port << "] is already in use"
-                   << std::endl;
+        LOG(ERROR) << "Could not create server TCP since the specified port[" << *port << "] is already in use";
         return 1;
       }
     }
@@ -343,6 +350,7 @@ int main(int argc, char *argv[]) {
   srv = &server;
   s = srv->Start();
   if (!s.IsOK()) {
+    LOG(ERROR) << "Failed to start server: " << s.Msg();
     return 1;
   }
   srv->Join();
