@@ -31,17 +31,21 @@
 #include <utility>
 
 #include "config.h"
+#include "fmt/format.h"
 #include "redis_connection.h"
 #include "redis_request.h"
 #include "storage/compaction_checker.h"
 #include "storage/redis_db.h"
 #include "storage/scripting.h"
+#include "string_util.h"
+#include "thread_util.h"
+#include "time_util.h"
 #include "tls_util.h"
-#include "util.h"
 #include "version.h"
 #include "worker.h"
 
 std::atomic<int> Server::unix_time_ = {0};
+constexpr const char *REDIS_VERSION = "4.0.0";
 
 Server::Server(Engine::Storage *storage, Config *config) : storage_(storage), config_(config) {
   // init commands stats here to prevent concurrent insert, and cause core
@@ -126,19 +130,25 @@ Status Server::Start() {
     if (!s.IsOK()) return s;
   } else {
     // Generate new replication id if not a replica
-    storage_->ShiftReplId();
+    auto s = storage_->ShiftReplId();
+    if (!s.IsOK()) {
+      return s.Prefixed("failed to shift replication id");
+    }
   }
 
-  if (config_->cluster_enabled) {
+  if (config_->cluster_enabled && config_->persist_cluster_nodes_enabled) {
+    auto s = cluster_->LoadClusterNodes(config_->NodesFilePath());
+    if (!s.IsOK()) {
+      return s.Prefixed("failed to load cluster nodes info");
+    }
     // Create objects used for slot migration
     slot_migrate_ =
         std::make_unique<SlotMigrate>(this, config_->migrate_speed, config_->pipeline_size, config_->sequence_gap);
     slot_import_ = new SlotImport(this);
     // Create migrating thread
-    auto s = slot_migrate_->CreateMigrateHandleThread();
+    s = slot_migrate_->CreateMigrateHandleThread();
     if (!s.IsOK()) {
-      LOG(ERROR) << "Failed to create migration thread, Err: " << s.Msg();
-      return Status(Status::NotOK);
+      return s.Prefixed("failed to create migration thread");
     }
   }
 
@@ -154,7 +164,7 @@ Status Server::Start() {
 
   compaction_checker_thread_ = std::thread([this]() {
     uint64_t counter = 0;
-    int32_t last_compact_date = 0;
+    time_t last_compact_date = 0;
     Util::ThreadSetName("compact-check");
     CompactionChecker compaction_checker(this->storage_);
     while (!stop_) {
@@ -212,7 +222,7 @@ void Server::Join() {
   if (compaction_checker_thread_.joinable()) compaction_checker_thread_.join();
 }
 
-Status Server::AddMaster(std::string host, uint32_t port, bool force_reconnect) {
+Status Server::AddMaster(const std::string &host, uint32_t port, bool force_reconnect) {
   std::lock_guard<std::mutex> guard(slaveof_mu_);
 
   // Don't check host and port if 'force_reconnect' argument is set to true
@@ -230,7 +240,7 @@ Status Server::AddMaster(std::string host, uint32_t port, bool force_reconnect)
   // replication, and uses 'listen-port + 1' as thread listening port.
   uint32_t master_listen_port = port;
   if (GetConfig()->master_use_repl_port) master_listen_port += 1;
-  replication_thread_ = std::unique_ptr<ReplicationThread>(new ReplicationThread(host, master_listen_port, this));
+  replication_thread_ = std::make_unique<ReplicationThread>(host, master_listen_port, this);
   auto s = replication_thread_->Start([this]() { PrepareRestoreDB(); },
                                       [this]() {
                                         this->is_loading_ = false;
@@ -252,9 +262,11 @@ Status Server::RemoveMaster() {
     master_host_.clear();
     master_port_ = 0;
     config_->ClearMaster();
-    if (replication_thread_) replication_thread_->Stop();
-    replication_thread_ = nullptr;
-    storage_->ShiftReplId();
+    if (replication_thread_) {
+      replication_thread_->Stop();
+      replication_thread_ = nullptr;
+    }
+    return storage_->ShiftReplId();
   }
   return Status::OK();
 }
@@ -403,7 +415,7 @@ void Server::GetChannelsByPattern(const std::string &pattern, std::vector<std::s
   }
 }
 
-void Server::ListChannelSubscribeNum(std::vector<std::string> channels,
+void Server::ListChannelSubscribeNum(const std::vector<std::string> &channels,
                                      std::vector<ChannelSubscribeNum> *channel_subscribe_nums) {
   std::lock_guard<std::mutex> guard(pubsub_channels_mu_);
   for (const auto &chan : channels) {
@@ -509,7 +521,7 @@ void Server::UnblockOnStreams(const std::vector<std::string> &keys, Redis::Conne
     }
 
     for (auto it = iter->second.begin(); it != iter->second.end();) {
-      auto consumer = *it;
+      const auto &consumer = *it;
       if (conn->GetFD() == consumer->fd && conn->Owner() == consumer->owner) {
         iter->second.erase(it);
         if (iter->second.empty()) {
@@ -517,44 +529,48 @@ void Server::UnblockOnStreams(const std::vector<std::string> &keys, Redis::Conne
         }
         break;
       }
+      ++it;
     }
   }
 }
 
-Status Server::WakeupBlockingConns(const std::string &key, size_t n_conns) {
+void Server::WakeupBlockingConns(const std::string &key, size_t n_conns) {
   std::lock_guard<std::mutex> guard(blocking_keys_mu_);
   auto iter = blocking_keys_.find(key);
   if (iter == blocking_keys_.end() || iter->second.empty()) {
-    return Status(Status::NotOK);
+    return;
   }
+
   while (n_conns-- && !iter->second.empty()) {
     auto conn_ctx = iter->second.front();
-    conn_ctx->owner->EnableWriteEvent(conn_ctx->fd);
+    auto s = conn_ctx->owner->EnableWriteEvent(conn_ctx->fd);
+    if (!s.IsOK()) {
+      LOG(ERROR) << "failed to enable write event on blocked client " << conn_ctx->fd << ": " << s.Msg();
+    }
     delConnContext(conn_ctx);
     iter->second.pop_front();
   }
-  return Status::OK();
 }
 
-Status Server::OnEntryAddedToStream(const std::string &ns, const std::string &key,
-                                    const Redis::StreamEntryID &entry_id) {
+void Server::OnEntryAddedToStream(const std::string &ns, const std::string &key, const Redis::StreamEntryID &entry_id) {
   std::lock_guard<std::mutex> guard(blocking_keys_mu_);
   auto iter = blocked_stream_consumers_.find(key);
   if (iter == blocked_stream_consumers_.end() || iter->second.empty()) {
-    return Status(Status::NotOK);
+    return;
   }
 
   for (auto it = iter->second.begin(); it != iter->second.end();) {
     auto consumer = *it;
     if (consumer->ns == ns && entry_id > consumer->last_consumed_id) {
-      consumer->owner->EnableWriteEvent(consumer->fd);
+      auto s = consumer->owner->EnableWriteEvent(consumer->fd);
+      if (!s.IsOK()) {
+        LOG(ERROR) << "failed to enable write event on blocked stream consumer " << consumer->fd << ": " << s.Msg();
+      }
       it = iter->second.erase(it);
     } else {
       ++it;
     }
   }
-
-  return Status::OK();
 }
 
 void Server::delConnContext(ConnContext *c) {
@@ -656,7 +672,7 @@ void Server::cron() {
       // Purge backup if needed, it will cost much disk space if we keep backup and full sync
       // checkpoints at the same time
       if (config_->purge_backup_on_fullsync && (storage_->ExistCheckpoint() || storage_->ExistSyncCheckpoint())) {
-        AsyncPurgeOldBackups(0, 0);
+        s = AsyncPurgeOldBackups(0, 0);
       }
     }
 
@@ -699,9 +715,9 @@ void Server::GetRocksDBInfo(std::string *info) {
   std::ostringstream string_stream;
   rocksdb::DB *db = storage_->GetDB();
 
-  uint64_t memtable_sizes, cur_memtable_sizes, num_snapshots, num_running_flushes;
-  uint64_t num_immutable_tables, memtable_flush_pending, compaction_pending;
-  uint64_t num_running_compaction, num_live_versions, num_superversion, num_backgroud_errors;
+  uint64_t memtable_sizes = 0, cur_memtable_sizes = 0, num_snapshots = 0, num_running_flushes = 0;
+  uint64_t num_immutable_tables = 0, memtable_flush_pending = 0, compaction_pending = 0;
+  uint64_t num_running_compaction = 0, num_live_versions = 0, num_superversion = 0, num_backgroud_errors = 0;
 
   db->GetAggregatedIntProperty("rocksdb.num-snapshots", &num_snapshots);
   db->GetAggregatedIntProperty("rocksdb.size-all-mem-tables", &memtable_sizes);
@@ -717,7 +733,7 @@ void Server::GetRocksDBInfo(std::string *info) {
 
   string_stream << "# RocksDB\r\n";
   for (const auto &cf_handle : *storage_->GetCFHandles()) {
-    uint64_t estimate_keys, block_cache_usage, block_cache_pinned_usage, index_and_filter_cache_usage;
+    uint64_t estimate_keys = 0, block_cache_usage = 0, block_cache_pinned_usage = 0, index_and_filter_cache_usage = 0;
     std::map<std::string, std::string> cf_stats_map;
     db->GetIntProperty(cf_handle, "rocksdb.estimate-num-keys", &estimate_keys);
     string_stream << "estimate_keys[" << cf_handle->GetName() << "]:" << estimate_keys << "\r\n";
@@ -769,7 +785,7 @@ void Server::GetRocksDBInfo(std::string *info) {
 }
 
 void Server::GetServerInfo(std::string *info) {
-  time_t now;
+  time_t now = 0;
   std::ostringstream string_stream;
   static int call_uname = 1;
   static utsname name;
@@ -781,6 +797,7 @@ void Server::GetServerInfo(std::string *info) {
   time(&now);
   string_stream << "# Server\r\n";
   string_stream << "version:" << VERSION << "\r\n";
+  string_stream << "redis_version:" << REDIS_VERSION << "\r\n";
   string_stream << "git_sha1:" << GIT_COMMIT << "\r\n";
   string_stream << "os:" << name.sysname << " " << name.release << " " << name.machine << "\r\n";
 #ifdef __GNUC__
@@ -808,11 +825,10 @@ void Server::GetClientsInfo(std::string *info) {
 
 void Server::GetMemoryInfo(std::string *info) {
   std::ostringstream string_stream;
-  char used_memory_rss_human[16], used_memory_lua_human[16];
   int64_t rss = Stats::GetMemoryRSS();
-  Util::BytesToHuman(used_memory_rss_human, 16, static_cast<uint64_t>(rss));
   int memory_lua = lua_gc(lua_, LUA_GCCOUNT, 0) * 1024;
-  Util::BytesToHuman(used_memory_lua_human, 16, static_cast<uint64_t>(memory_lua));
+  std::string used_memory_rss_human = Util::BytesToHuman(rss);
+  std::string used_memory_lua_human = Util::BytesToHuman(memory_lua);
   string_stream << "# Memory\r\n";
   string_stream << "used_memory_rss:" << rss << "\r\n";
   string_stream << "used_memory_human:" << used_memory_rss_human << "\r\n";
@@ -823,7 +839,7 @@ void Server::GetMemoryInfo(std::string *info) {
 }
 
 void Server::GetReplicationInfo(std::string *info) {
-  time_t now;
+  time_t now = 0;
   std::ostringstream string_stream;
   string_stream << "# Replication\r\n";
   string_stream << "role:" << (IsSlave() ? "slave" : "master") << "\r\n";
@@ -964,23 +980,28 @@ void Server::GetInfo(const std::string &ns, const std::string &section, std::str
   info->clear();
   std::ostringstream string_stream;
   bool all = section == "all";
+  int section_cnt = 0;
 
   if (all || section == "server") {
     std::string server_info;
     GetServerInfo(&server_info);
+    if (section_cnt++) string_stream << "\r\n";
     string_stream << server_info;
   }
   if (all || section == "clients") {
     std::string clients_info;
     GetClientsInfo(&clients_info);
+    if (section_cnt++) string_stream << "\r\n";
     string_stream << clients_info;
   }
   if (all || section == "memory") {
     std::string memory_info;
     GetMemoryInfo(&memory_info);
+    if (section_cnt++) string_stream << "\r\n";
     string_stream << memory_info;
   }
   if (all || section == "persistence") {
+    if (section_cnt++) string_stream << "\r\n";
     string_stream << "# Persistence\r\n";
     string_stream << "loading:" << is_loading_ << "\r\n";
 
@@ -993,6 +1014,7 @@ void Server::GetInfo(const std::string &ns, const std::string &section, std::str
   if (all || section == "stats") {
     std::string stats_info;
     GetStatsInfo(&stats_info);
+    if (section_cnt++) string_stream << "\r\n";
     string_stream << stats_info;
   }
 
@@ -1000,11 +1022,13 @@ void Server::GetInfo(const std::string &ns, const std::string &section, std::str
   if (!is_loading_ && (all || section == "replication")) {
     std::string replication_info;
     GetReplicationInfo(&replication_info);
+    if (section_cnt++) string_stream << "\r\n";
     string_stream << replication_info;
   }
   if (all || section == "cpu") {
     struct rusage self_ru;
     getrusage(RUSAGE_SELF, &self_ru);
+    if (section_cnt++) string_stream << "\r\n";
     string_stream << "# CPU\r\n";
     string_stream << "used_cpu_sys:"
                   << static_cast<float>(self_ru.ru_stime.tv_sec) +
@@ -1018,6 +1042,7 @@ void Server::GetInfo(const std::string &ns, const std::string &section, std::str
   if (all || section == "commandstats") {
     std::string commands_stats_info;
     GetCommandsStatsInfo(&commands_stats_info);
+    if (section_cnt++) string_stream << "\r\n";
     string_stream << commands_stats_info;
   }
 
@@ -1026,6 +1051,7 @@ void Server::GetInfo(const std::string &ns, const std::string &section, std::str
     KeyNumStats stats;
     GetLastestKeyNumStats(ns, &stats);
     time_t last_scan_time = GetLastScanTime(ns);
+    if (section_cnt++) string_stream << "\r\n";
     string_stream << "# Keyspace\r\n";
     string_stream << "# Last scan db time: " << std::asctime(std::localtime(&last_scan_time));
     string_stream << "db0:keys=" << stats.n_key << ",expires=" << stats.n_expires << ",avg_ttl=" << stats.avg_ttl
@@ -1033,7 +1059,9 @@ void Server::GetInfo(const std::string &ns, const std::string &section, std::str
     string_stream << "sequence:" << storage_->GetDB()->GetLatestSequenceNumber() << "\r\n";
     string_stream << "used_db_size:" << storage_->GetTotalSize(ns) << "\r\n";
     string_stream << "max_db_size:" << config_->max_db_size * GiB << "\r\n";
-    double used_percent = config_->max_db_size ? storage_->GetTotalSize() * 100 / (config_->max_db_size * GiB) : 0;
+    double used_percent = config_->max_db_size ? static_cast<double>(storage_->GetTotalSize() * 100) /
+                                                     static_cast<double>(config_->max_db_size * GiB)
+                                               : 0;
     string_stream << "used_percent: " << used_percent << "%\r\n";
     struct statvfs stat;
     if (statvfs(config_->db_dir.c_str(), &stat) == 0) {
@@ -1041,7 +1069,7 @@ void Server::GetInfo(const std::string &ns, const std::string &section, std::str
       auto used_disk_size = (stat.f_blocks - stat.f_bavail) * stat.f_frsize;
       string_stream << "disk_capacity:" << disk_capacity << "\r\n";
       string_stream << "used_disk_size:" << used_disk_size << "\r\n";
-      double used_disk_percent = used_disk_size * 100 / disk_capacity;
+      double used_disk_percent = static_cast<double>(used_disk_size * 100) / static_cast<double>(disk_capacity);
       string_stream << "used_disk_percent: " << used_disk_percent << "%\r\n";
     }
   }
@@ -1050,29 +1078,27 @@ void Server::GetInfo(const std::string &ns, const std::string &section, std::str
   if (!is_loading_ && (all || section == "rocksdb")) {
     std::string rocksdb_info;
     GetRocksDBInfo(&rocksdb_info);
+    if (section_cnt++) string_stream << "\r\n";
     string_stream << rocksdb_info;
   }
   *info = string_stream.str();
 }
 
 std::string Server::GetRocksDBStatsJson() {
-  char buf[256];
   std::string output;
 
   output.reserve(8 * 1024);
   output.append("{");
   auto stats = storage_->GetDB()->GetDBOptions().statistics;
   for (const auto &iter : rocksdb::TickersNameMap) {
-    snprintf(buf, sizeof(buf), "\"%s\":%" PRIu64 ",", iter.second.c_str(), stats->getTickerCount(iter.first));
-    output.append(buf);
+    output.append(fmt::format(R"("{}":{},)", iter.second.c_str(), stats->getTickerCount(iter.first)));
   }
   for (const auto &iter : rocksdb::HistogramsNameMap) {
     rocksdb::HistogramData hist_data;
     stats->histogramData(iter.first, &hist_data);
     /* P50 P95 P99 P100 COUNT SUM */
-    snprintf(buf, sizeof(buf), "\"%s\":[%f,%f,%f,%f,%" PRIu64 ",%" PRIu64 "],", iter.second.c_str(), hist_data.median,
-             hist_data.percentile95, hist_data.percentile99, hist_data.max, hist_data.count, hist_data.sum);
-    output.append(buf);
+    output.append(fmt::format(R"("{}":[{},{},{},{},{},{}],)", iter.second, hist_data.median, hist_data.percentile95,
+                              hist_data.percentile99, hist_data.max, hist_data.count, hist_data.sum));
   }
   output.pop_back();
   output.append("}");
@@ -1295,14 +1321,14 @@ void Server::SlowlogPushEntryIfNeeded(const std::vector<std::string> *args, uint
   size_t argc = args->size() > kSlowLogMaxArgc ? kSlowLogMaxArgc : args->size();
   for (size_t i = 0; i < argc; i++) {
     if (argc != args->size() && i == argc - 1) {
-      entry->args.emplace_back("... (" + std::to_string(args->size() - argc + 1) + " more arguments)");
+      entry->args.emplace_back(fmt::format("... ({} more arguments)", args->size() - argc + 1));
       break;
     }
     if (args->data()[i].length() <= kSlowLogMaxString) {
       entry->args.emplace_back(args->data()[i]);
     } else {
-      entry->args.emplace_back(args->data()[i].substr(0, kSlowLogMaxString) + "... (" +
-                               std::to_string(args->data()[i].length() - kSlowLogMaxString) + " more bytes)");
+      entry->args.emplace_back(fmt::format("{}... ({} more bytes)", args->data()[i].substr(0, kSlowLogMaxString),
+                                           args->data()[i].length() - kSlowLogMaxString));
     }
   }
   entry->duration = duration;
@@ -1321,7 +1347,7 @@ std::string Server::GetClientsStr() {
   return clients;
 }
 
-void Server::KillClient(int64_t *killed, std::string addr, uint64_t id, uint64_t type, bool skipme,
+void Server::KillClient(int64_t *killed, const std::string &addr, uint64_t id, uint64_t type, bool skipme,
                         Redis::Connection *conn) {
   *killed = 0;
 
@@ -1347,7 +1373,9 @@ void Server::KillClient(int64_t *killed, std::string addr, uint64_t id, uint64_t
   if (IsSlave() &&
       (type & kTypeMaster || (!addr.empty() && addr == master_host_ + ":" + std::to_string(master_port_)))) {
     // Stop replication thread and start a new one to replicate
-    AddMaster(master_host_, master_port_, true);
+    if (auto s = AddMaster(master_host_, master_port_, true); !s.IsOK()) {
+      LOG(ERROR) << "Failed to add master " << master_host_ << ":" << master_port_ << "with error: " << s.Msg();
+    }
     (*killed)++;
   }
 }
@@ -1388,9 +1416,9 @@ Status Server::ScriptGet(const std::string &sha, std::string *body) {
   return Status::OK();
 }
 
-void Server::ScriptSet(const std::string &sha, const std::string &body) {
+Status Server::ScriptSet(const std::string &sha, const std::string &body) {
   std::string funcname = Engine::kLuaFunctionPrefix + sha;
-  storage_->WriteToPropagateCF(funcname, body);
+  return storage_->WriteToPropagateCF(funcname, body);
 }
 
 void Server::ScriptReset() {
