@@ -28,7 +28,6 @@
 #include "parse_util.h"
 #include "rocksdb/iterator.h"
 #include "server/server.h"
-#include "util.h"
 
 namespace Redis {
 
@@ -163,7 +162,7 @@ rocksdb::Status Database::TTL(const Slice &user_key, int *ttl) {
 
 void Database::GetKeyNumStats(const std::string &prefix, KeyNumStats *stats) { Keys(prefix, nullptr, stats); }
 
-void Database::Keys(std::string prefix, std::vector<std::string> *keys, KeyNumStats *stats) {
+void Database::Keys(const std::string &prefix, std::vector<std::string> *keys, KeyNumStats *stats) {
   uint16_t slot_id = 0;
   std::string ns_prefix, ns, user_key, value;
   if (namespace_ != kDefaultNamespace || keys != nullptr) {
@@ -182,7 +181,7 @@ void Database::Keys(std::string prefix, std::vector<std::string> *keys, KeyNumSt
   LatestSnapShot ss(db_);
   rocksdb::ReadOptions read_options;
   read_options.snapshot = ss.GetSnapShot();
-  read_options.fill_cache = false;
+  storage_->SetReadOptions(read_options);
   auto iter = DBUtil::UniqueIterator(db_, read_options, metadata_cf_handle_);
 
   while (true) {
@@ -236,7 +235,7 @@ rocksdb::Status Database::Scan(const std::string &cursor, uint64_t limit, const
   LatestSnapShot ss(db_);
   rocksdb::ReadOptions read_options;
   read_options.snapshot = ss.GetSnapShot();
-  read_options.fill_cache = false;
+  storage_->SetReadOptions(read_options);
   auto iter = DBUtil::UniqueIterator(db_, read_options, metadata_cf_handle_);
 
   AppendNamespacePrefix(cursor, &ns_cursor);
@@ -335,7 +334,7 @@ rocksdb::Status Database::RandomKey(const std::string &cursor, std::string *key)
     }
   }
   if (!keys.empty()) {
-    unsigned int seed = time(NULL);
+    unsigned int seed = time(nullptr);
     *key = keys.at(rand_r(&seed) % keys.size());
   }
   return rocksdb::Status::OK();
@@ -360,7 +359,7 @@ rocksdb::Status Database::FlushAll() {
   LatestSnapShot ss(db_);
   rocksdb::ReadOptions read_options;
   read_options.snapshot = ss.GetSnapShot();
-  read_options.fill_cache = false;
+  storage_->SetReadOptions(read_options);
   auto iter = DBUtil::UniqueIterator(db_, read_options, metadata_cf_handle_);
   iter->SeekToFirst();
   if (!iter->Valid()) {
@@ -466,7 +465,7 @@ rocksdb::Status Database::FindKeyRangeWithPrefix(const std::string &prefix, cons
   LatestSnapShot ss(storage_->GetDB());
   rocksdb::ReadOptions read_options;
   read_options.snapshot = ss.GetSnapShot();
-  read_options.fill_cache = false;
+  storage_->SetReadOptions(read_options);
   auto iter = DBUtil::UniqueIterator(storage_->GetDB(), read_options, cf_handle);
   iter->Seek(prefix);
   if (!iter->Valid() || !iter->key().starts_with(prefix)) {
@@ -517,11 +516,11 @@ rocksdb::Status Database::ClearKeysOfSlot(const rocksdb::Slice &ns, int slot) {
 
 rocksdb::Status Database::GetSlotKeysInfo(int slot, std::map<int, uint64_t> *slotskeys, std::vector<std::string> *keys,
                                           int count) {
-  const rocksdb::Snapshot *snapshot;
+  const rocksdb::Snapshot *snapshot = nullptr;
   snapshot = storage_->GetDB()->GetSnapshot();
   rocksdb::ReadOptions read_options;
   read_options.snapshot = snapshot;
-  read_options.fill_cache = false;
+  storage_->SetReadOptions(read_options);
   auto iter = db_->NewIterator(read_options, metadata_cf_handle_);
   bool end = false;
   for (int i = 0; i < HASH_SLOTS_SIZE; i++) {
@@ -568,7 +567,7 @@ rocksdb::Status SubKeyScanner::Scan(RedisType type, const Slice &user_key, const
   LatestSnapShot ss(db_);
   rocksdb::ReadOptions read_options;
   read_options.snapshot = ss.GetSnapShot();
-  read_options.fill_cache = false;
+  storage_->SetReadOptions(read_options);
   auto iter = DBUtil::UniqueIterator(db_, read_options);
   std::string match_prefix_key;
   if (!subkey_prefix.empty()) {
@@ -611,8 +610,8 @@ std::vector<std::string> *WriteBatchLogData::GetArguments() { return &args_; }
 
 std::string WriteBatchLogData::Encode() {
   std::string ret = std::to_string(type_);
-  for (size_t i = 0; i < args_.size(); i++) {
-    ret += " " + args_[i];
+  for (const auto &arg : args_) {
+    ret += " " + arg;
   }
   return ret;
 }
