@@ -18,35 +18,6 @@
  *
  */
 
-/*
- * Copyright (c) 2009-2012, Salvatore Sanfilippo <antirez at gmail dot com>
- * All rights reserved.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions are met:
- *
- *   * Redistributions of source code must retain the above copyright notice,
- *     this list of conditions and the following disclaimer.
- *   * Redistributions in binary form must reproduce the above copyright
- *     notice, this list of conditions and the following disclaimer in the
- *     documentation and/or other materials provided with the distribution.
- *   * Neither the name of Redis nor the names of its contributors may be used
- *     to endorse or promote products derived from this software without
- *     specific prior written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
- * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
- * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
- * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
- * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
- * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
- * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
- * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
- * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
- * POSSIBILITY OF SUCH DAMAGE.
- */
-
 // This file is modified from several source code files about lua scripting of Redis.
 // See the original code at https://github.com/redis/redis.
 
@@ -57,16 +28,17 @@
 #include <string>
 
 #include "commands/redis_cmd.h"
+#include "fmt/format.h"
+#include "parse_util.h"
 #include "rand.h"
 #include "server/redis_connection.h"
 #include "server/server.h"
 #include "sha1.h"
-#include "util.h"
 
 /* The maximum number of characters needed to represent a long double
  * as a string (long double has a huge range).
  * This should be the size of the buffer given to doule to string */
-#define MAX_LONG_DOUBLE_CHARS 5 * 1024
+constexpr size_t MAX_LONG_DOUBLE_CHARS = 5 * 1024;
 
 enum {
   LL_DEBUG = 0,
@@ -193,7 +165,7 @@ void loadFuncs(lua_State *lua, bool read_only) {
 }
 
 int redisLogCommand(lua_State *lua) {
-  int j, level, argc = lua_gettop(lua);
+  int j = 0, level = 0, argc = lua_gettop(lua);
 
   if (argc < 2) {
     lua_pushstring(lua, "redis.log() requires two arguments or more.");
@@ -203,19 +175,19 @@ int redisLogCommand(lua_State *lua) {
     lua_pushstring(lua, "First argument must be a number (log level).");
     return lua_error(lua);
   }
-  level = lua_tonumber(lua, -argc);
+  level = static_cast<int>(lua_tonumber(lua, -argc));
   if (level < LL_DEBUG || level > LL_WARNING) {
     lua_pushstring(lua, "Invalid debug level.");
     return lua_error(lua);
   }
-  if (level < GetServer()->GetConfig()->loglevel) {
+  if (level < GetServer()->GetConfig()->log_level) {
     return 0;
   }
 
   std::string log_message;
   for (j = 1; j < argc; j++) {
-    size_t len;
-    const char *s;
+    size_t len = 0;
+    const char *s = nullptr;
     s = lua_tolstring(lua, (-argc) + j, &len);
     if (s) {
       if (j != 1) {
@@ -250,14 +222,11 @@ Status evalGenericCommand(Redis::Connection *conn, const std::vector<std::string
     lua = conn->Owner()->Lua();
   }
 
-  auto s = Util::DecimalStringToNum(args[2], &numkeys);
-  if (!s.IsOK()) {
-    return s;
-  }
+  numkeys = GET_OR_RET(ParseInt<int64_t>(args[2], 10));
   if (numkeys > int64_t(args.size() - 3)) {
-    return Status(Status::NotOK, "Number of keys can't be greater than number of args");
+    return {Status::NotOK, "Number of keys can't be greater than number of args"};
   } else if (numkeys < -1) {
-    return Status(Status::NotOK, "Number of keys can't be negative");
+    return {Status::NotOK, "Number of keys can't be negative"};
   }
 
   /* We obtain the script SHA1, then check if this function is already
@@ -269,7 +238,7 @@ Status evalGenericCommand(Redis::Connection *conn, const std::vector<std::string
   } else {
     for (int j = 0; j < 40; j++) {
       std::string sha = args[1];
-      funcname[j + 2] = (sha[j] >= 'A' && sha[j] <= 'Z') ? sha[j] + ('a' - 'A') : sha[j];
+      funcname[j + 2] = (sha[j] >= 'A' && sha[j] <= 'Z') ? static_cast<char>(sha[j] + 'a' - 'A') : sha[j];
     }
     funcname[42] = '\0';
   }
@@ -286,13 +255,13 @@ Status evalGenericCommand(Redis::Connection *conn, const std::vector<std::string
       auto s = srv->ScriptGet(funcname + 2, &body);
       if (!s.IsOK()) {
         lua_pop(lua, 1); /* remove the error handler from the stack. */
-        return Status(Status::NotOK, "NOSCRIPT No matching script. Please use EVAL");
+        return {Status::NotOK, "NOSCRIPT No matching script. Please use EVAL"};
       }
     } else {
       body = args[1];
     }
     std::string sha;
-    s = createFunction(srv, body, &sha, lua);
+    auto s = createFunction(srv, body, &sha, lua);
     if (!s.IsOK()) {
       lua_pop(lua, 1); /* remove the error handler from the stack. */
       return s;
@@ -315,13 +284,13 @@ Status evalGenericCommand(Redis::Connection *conn, const std::vector<std::string
     lua_pop(lua, 1);
   }
 
-/* Call the Lua garbage collector from time to time to avoid a
- * full cycle performed by Lua, which adds too latency.
- *
- * The call is performed every LUA_GC_CYCLE_PERIOD executed commands
- * (and for LUA_GC_CYCLE_PERIOD collection steps) because calling it
- * for every command uses too much CPU. */
-#define LUA_GC_CYCLE_PERIOD 50
+  /* Call the Lua garbage collector from time to time to avoid a
+   * full cycle performed by Lua, which adds too latency.
+   *
+   * The call is performed every LUA_GC_CYCLE_PERIOD executed commands
+   * (and for LUA_GC_CYCLE_PERIOD collection steps) because calling it
+   * for every command uses too much CPU. */
+  constexpr int64_t LUA_GC_CYCLE_PERIOD = 50;
   {
     static int64_t gc_count = 0;
 
@@ -338,7 +307,7 @@ int redisCallCommand(lua_State *lua) { return redisGenericCommand(lua, 1); }
 
 int redisPCallCommand(lua_State *lua) { return redisGenericCommand(lua, 0); }
 int redisGenericCommand(lua_State *lua, int raise_error) {
-  int j, argc = lua_gettop(lua);
+  int j = 0, argc = lua_gettop(lua);
   std::vector<std::string> args;
   lua_getglobal(lua, "redis");
   lua_getfield(lua, -1, "read_only");
@@ -350,17 +319,14 @@ int redisGenericCommand(lua_State *lua, int raise_error) {
     return raise_error ? raiseError(lua) : 1;
   }
   for (j = 0; j < argc; j++) {
-    char dbuf[64];
     if (lua_type(lua, j + 1) == LUA_TNUMBER) {
       lua_Number num = lua_tonumber(lua, j + 1);
-      snprintf(dbuf, sizeof(dbuf), "%.17g", static_cast<double>(num));
-      args.emplace_back(dbuf);
+      args.emplace_back(fmt::format("{:.17g}", static_cast<double>(num)));
     } else {
-      const char *obj_s;
-      size_t obj_len;
-      obj_s = lua_tolstring(lua, j + 1, &obj_len);
+      size_t obj_len = 0;
+      const char *obj_s = lua_tolstring(lua, j + 1, &obj_len);
       if (obj_s == nullptr) break; /* no a string */
-      args.emplace_back(std::string(obj_s, obj_len));
+      args.emplace_back(obj_s, obj_len);
     }
   }
   if (j != argc) {
@@ -519,8 +485,8 @@ int redisStatusReplyCommand(lua_State *lua) { return redisReturnSingleFieldTable
 int redisSha1hexCommand(lua_State *lua) {
   int argc = lua_gettop(lua);
   char digest[41];
-  size_t len;
-  const char *s;
+  size_t len = 0;
+  const char *s = nullptr;
 
   if (argc != 1) {
     lua_pushstring(lua, "wrong number of arguments");
@@ -547,7 +513,7 @@ void SHA1Hex(char *digest, const char *script, size_t len) {
   SHA1_CTX ctx;
   unsigned char hash[20];
   const char *cset = "0123456789abcdef";
-  int j;
+  int j = 0;
 
   SHA1Init(&ctx);
   SHA1Update(&ctx, (const unsigned char *)script, len);
@@ -622,18 +588,15 @@ const char *redisProtocolToLuaType(lua_State *lua, const char *reply) {
 
 const char *redisProtocolToLuaType_Int(lua_State *lua, const char *reply) {
   const char *p = strchr(reply + 1, '\r');
-  int64_t value;
-
-  Util::DecimalStringToNum(std::string(reply + 1, p - reply - 1), &value);
+  auto value = ParseInt<int64_t>(std::string(reply + 1, p - reply - 1), 10).ValueOr(0);
   lua_pushnumber(lua, static_cast<lua_Number>(value));
   return p + 2;
 }
 
 const char *redisProtocolToLuaType_Bulk(lua_State *lua, const char *reply) {
   const char *p = strchr(reply + 1, '\r');
-  int64_t bulklen;
+  auto bulklen = ParseInt<int64_t>(std::string(reply + 1, p - reply - 1), 10).ValueOr(0);
 
-  Util::DecimalStringToNum(std::string(reply + 1, p - reply - 1), &bulklen);
   if (bulklen == -1) {
     lua_pushboolean(lua, 0);
     return p + 2;
@@ -665,10 +628,9 @@ const char *redisProtocolToLuaType_Error(lua_State *lua, const char *reply) {
 
 const char *redisProtocolToLuaType_Aggregate(lua_State *lua, const char *reply, int atype) {
   const char *p = strchr(reply + 1, '\r');
-  int64_t mbulklen;
+  int64_t mbulklen = ParseInt<int64_t>(std::string(reply + 1, p - reply - 1), 10).ValueOr(0);
   int j = 0;
 
-  Util::DecimalStringToNum(std::string(reply + 1, p - reply - 1), &mbulklen);
   p += 2;
   if (mbulklen == -1) {
     lua_pushboolean(lua, 0);
@@ -699,7 +661,7 @@ const char *redisProtocolToLuaType_Double(lua_State *lua, const char *reply) {
   const char *p = strchr(reply + 1, '\r');
   char buf[MAX_LONG_DOUBLE_CHARS + 1];
   size_t len = p - reply - 1;
-  double d;
+  double d = NAN;
 
   if (len <= MAX_LONG_DOUBLE_CHARS) {
     memcpy(buf, reply + 1, len);
@@ -731,7 +693,7 @@ void pushError(lua_State *lua, const char *err) {
 std::string replyToRedisReply(lua_State *lua) {
   std::string output;
   const char *obj_s = nullptr;
-  size_t obj_len;
+  size_t obj_len = 0;
 
   int t = lua_type(lua, -1);
   switch (t) {
@@ -840,7 +802,7 @@ void setGlobalArray(lua_State *lua, const std::string &var, const std::vector<st
   lua_newtable(lua);
   for (size_t i = 0; i < elems.size(); i++) {
     lua_pushlstring(lua, elems[i].c_str(), elems[i].size());
-    lua_rawseti(lua, -2, i + 1);
+    lua_rawseti(lua, -2, static_cast<int>(i) + 1);
   }
   lua_setglobal(lua, var.c_str());
 }
@@ -935,8 +897,7 @@ Status createFunction(Server *srv, const std::string &body, std::string *sha, lu
     return Status(Status::NotOK, "Error running script (new function): " + errMsg + "\n");
   }
   // would store lua function into propagate column family and propagate those scripts to slaves
-  srv->ScriptSet(*sha, body);
-  return Status::OK();
+  return srv->ScriptSet(*sha, body);
 }
 
 }  // namespace Lua
