@@ -47,6 +47,7 @@
 #include "rocksdb_crc32c.h"
 #include "server/server.h"
 #include "table_properties_collector.h"
+#include "time_util.h"
 
 namespace Engine {
 
@@ -63,7 +64,7 @@ const char *kLuaFunctionPrefix = "lua_f_";
 
 const char *kReplicationIdKey = "replication_id_";
 
-const uint64_t kIORateLimitMaxMb = 1024000;
+const int64_t kIORateLimitMaxMb = 1024000;
 
 using rocksdb::Slice;
 
@@ -76,7 +77,7 @@ Storage::Storage(Config *config) : env_(rocksdb::Env::Default()), config_(config
 }
 
 Storage::~Storage() {
-  if (backup_ != nullptr) {
+  if (backup_) {
     DestroyBackup();
   }
   CloseDB();
@@ -102,6 +103,11 @@ void Storage::SetWriteOptions(const Config::RocksDB::WriteOptions &config) {
   write_opts_.memtable_insert_hint_per_batch = config.memtable_insert_hint_per_batch;
 }
 
+void Storage::SetReadOptions(rocksdb::ReadOptions &read_options) {
+  read_options.fill_cache = false;
+  read_options.async_io = config_->RocksDB.read_options.async_io;
+}
+
 rocksdb::BlockBasedTableOptions Storage::InitTableOptions() {
   rocksdb::BlockBasedTableOptions table_options;
   table_options.format_version = 5;
@@ -168,9 +174,10 @@ rocksdb::Options Storage::InitOptions() {
   options.dump_malloc_stats = true;
   sst_file_manager_ = std::shared_ptr<rocksdb::SstFileManager>(rocksdb::NewSstFileManager(rocksdb::Env::Default()));
   options.sst_file_manager = sst_file_manager_;
-  uint64_t max_io_mb = kIORateLimitMaxMb;
-  if (config_->max_io_mb > 0) max_io_mb = static_cast<uint64_t>(config_->max_io_mb);
-  rate_limiter_ = std::shared_ptr<rocksdb::RateLimiter>(rocksdb::NewGenericRateLimiter(max_io_mb * MiB));
+  int64_t max_io_mb = kIORateLimitMaxMb;
+  if (config_->max_io_mb > 0) max_io_mb = config_->max_io_mb;
+  rate_limiter_ =
+      std::shared_ptr<rocksdb::RateLimiter>(rocksdb::NewGenericRateLimiter(max_io_mb * static_cast<int64_t>(MiB)));
   options.rate_limiter = rate_limiter_;
   options.delayed_write_rate = static_cast<uint64_t>(config_->RocksDB.delayed_write_rate);
   options.compaction_readahead_size = static_cast<size_t>(config_->RocksDB.compaction_readahead_size);
@@ -205,7 +212,7 @@ Status Storage::SetDBOption(const std::string &key, const std::string &value) {
 }
 
 Status Storage::CreateColumnFamilies(const rocksdb::Options &options) {
-  rocksdb::DB *tmp_db;
+  rocksdb::DB *tmp_db = nullptr;
   rocksdb::ColumnFamilyOptions cf_options(options);
   rocksdb::Status s = rocksdb::DB::Open(options, config_->db_dir, &tmp_db);
   if (s.ok()) {
@@ -246,7 +253,9 @@ Status Storage::Open(bool read_only) {
   size_t subkey_block_cache_size = config_->RocksDB.subkey_block_cache_size * MiB;
 
   rocksdb::Options options = InitOptions();
-  CreateColumnFamilies(options);
+  if (auto s = CreateColumnFamilies(options); !s.IsOK()) {
+    return s.Prefixed("failed to create column families");
+  }
 
   std::shared_ptr<rocksdb::Cache> shared_block_cache;
   if (config_->RocksDB.share_metadata_and_subkey_block_cache) {
@@ -302,12 +311,12 @@ Status Storage::Open(bool read_only) {
 
   std::vector<rocksdb::ColumnFamilyDescriptor> column_families;
   // Caution: don't change the order of column family, or the handle will be mismatched
-  column_families.emplace_back(rocksdb::ColumnFamilyDescriptor(rocksdb::kDefaultColumnFamilyName, subkey_opts));
-  column_families.emplace_back(rocksdb::ColumnFamilyDescriptor(kMetadataColumnFamilyName, metadata_opts));
-  column_families.emplace_back(rocksdb::ColumnFamilyDescriptor(kZSetScoreColumnFamilyName, subkey_opts));
-  column_families.emplace_back(rocksdb::ColumnFamilyDescriptor(kPubSubColumnFamilyName, pubsub_opts));
-  column_families.emplace_back(rocksdb::ColumnFamilyDescriptor(kPropagateColumnFamilyName, propagate_opts));
-  column_families.emplace_back(rocksdb::ColumnFamilyDescriptor(kStreamColumnFamilyName, subkey_opts));
+  column_families.emplace_back(rocksdb::kDefaultColumnFamilyName, subkey_opts);
+  column_families.emplace_back(kMetadataColumnFamilyName, metadata_opts);
+  column_families.emplace_back(kZSetScoreColumnFamilyName, subkey_opts);
+  column_families.emplace_back(kPubSubColumnFamilyName, pubsub_opts);
+  column_families.emplace_back(kPropagateColumnFamilyName, propagate_opts);
+  column_families.emplace_back(kStreamColumnFamilyName, subkey_opts);
   std::vector<std::string> old_column_families;
   auto s = rocksdb::DB::ListColumnFamilies(options, config_->db_dir, &old_column_families);
   if (!s.ok()) return Status(Status::NotOK, s.ToString());
@@ -337,7 +346,7 @@ Status Storage::CreateBackup() {
   rocksdb::DestroyDB(tmpdir, rocksdb::Options());
 
   // 1) Create checkpoint of rocksdb for backup
-  rocksdb::Checkpoint *checkpoint = NULL;
+  rocksdb::Checkpoint *checkpoint = nullptr;
   rocksdb::Status s = rocksdb::Checkpoint::Create(db_, &checkpoint);
   if (!s.ok()) {
     LOG(WARNING) << "Fail to create checkpoint for backup, error:" << s.ToString();
@@ -370,10 +379,9 @@ Status Storage::CreateBackup() {
   return Status::OK();
 }
 
-Status Storage::DestroyBackup() {
+void Storage::DestroyBackup() {
   backup_->StopBackup();
   delete backup_;
-  return Status();
 }
 
 Status Storage::RestoreFromBackup() {
@@ -585,8 +593,8 @@ uint64_t Storage::GetTotalSize(const std::string &ns) {
   ComposeNamespaceKey(ns, "", &prefix, false);
 
   Redis::Database db(this, ns);
-  uint64_t size, total_size = 0;
-  uint8_t include_both =
+  uint64_t size = 0, total_size = 0;
+  rocksdb::DB::SizeApproximationFlags include_both =
       rocksdb::DB::SizeApproximationFlags::INCLUDE_FILES | rocksdb::DB::SizeApproximationFlags::INCLUDE_MEMTABLES;
   for (auto cf_handle : cf_handles_) {
     if (cf_handle == GetCFHandle(kPubSubColumnFamilyName) || cf_handle == GetCFHandle(kPropagateColumnFamilyName)) {
@@ -602,16 +610,17 @@ uint64_t Storage::GetTotalSize(const std::string &ns) {
   return total_size;
 }
 
-Status Storage::CheckDBSizeLimit() {
-  bool reach_db_size_limit;
+void Storage::CheckDBSizeLimit() {
+  bool reach_db_size_limit = false;
   if (config_->max_db_size == 0) {
     reach_db_size_limit = false;
   } else {
     reach_db_size_limit = GetTotalSize() >= config_->max_db_size * GiB;
   }
   if (reach_db_size_limit_ == reach_db_size_limit) {
-    return Status::OK();
+    return;
   }
+
   reach_db_size_limit_ = reach_db_size_limit;
   if (reach_db_size_limit_) {
     LOG(WARNING) << "[storage] ENABLE db_size limit " << config_->max_db_size << " GB"
@@ -619,14 +628,13 @@ Status Storage::CheckDBSizeLimit() {
   } else {
     LOG(WARNING) << "[storage] DISABLE db_size limit, set kvrocks to read-write mode ";
   }
-  return Status::OK();
 }
 
-void Storage::SetIORateLimit(uint64_t max_io_mb) {
+void Storage::SetIORateLimit(int64_t max_io_mb) {
   if (max_io_mb == 0) {
     max_io_mb = kIORateLimitMaxMb;
   }
-  rate_limiter_->SetBytesPerSecond(max_io_mb * MiB);
+  rate_limiter_->SetBytesPerSecond(max_io_mb * static_cast<int64_t>(MiB));
 }
 
 rocksdb::DB *Storage::GetDB() { return db_; }
@@ -643,12 +651,12 @@ Status Storage::WriteToPropagateCF(const std::string &key, const std::string &va
   return Status::OK();
 }
 
-bool Storage::ShiftReplId(void) {
+Status Storage::ShiftReplId() {
   const char *charset = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ";
-  const int charset_len = strlen(charset);
+  const int charset_len = static_cast<int>(strlen(charset));
 
   // Do nothing if don't enable rsid psync
-  if (!config_->use_rsid_psync) return true;
+  if (!config_->use_rsid_psync) return Status::OK();
 
   std::random_device rd;
   std::mt19937 gen(rd() + getpid());
@@ -661,8 +669,7 @@ bool Storage::ShiftReplId(void) {
   LOG(INFO) << "[replication] New replication id: " << replid_;
 
   // Write new replication id into db engine
-  WriteToPropagateCF(kReplicationIdKey, replid_);
-  return true;
+  return WriteToPropagateCF(kReplicationIdKey, replid_);
 }
 
 std::string Storage::GetReplIdFromWalBySeq(rocksdb::SequenceNumber seq) {
@@ -695,7 +702,7 @@ std::string Storage::GetReplIdFromWalBySeq(rocksdb::SequenceNumber seq) {
         }
       }
     };
-    std::string GetReplId(void) { return replid_in_wal_; }
+    std::string GetReplId() { return replid_in_wal_; }
 
    private:
     std::string replid_in_wal_;
@@ -708,7 +715,7 @@ std::string Storage::GetReplIdFromWalBySeq(rocksdb::SequenceNumber seq) {
   return write_batch_handler.GetReplId();
 }
 
-std::string Storage::GetReplIdFromDbEngine(void) {
+std::string Storage::GetReplIdFromDbEngine() {
   std::string replid_in_db;
   auto cf = GetCFHandle(kPropagateColumnFamilyName);
   auto s = db_->Get(rocksdb::ReadOptions(), cf, kReplicationIdKey, &replid_in_db);
@@ -730,7 +737,7 @@ Status Storage::ReplDataManager::GetFullReplDataInfo(Storage *storage, std::stri
 
   // Create checkpoint if not exist
   if (!storage->env_->FileExists(data_files_dir).ok()) {
-    rocksdb::Checkpoint *checkpoint = NULL;
+    rocksdb::Checkpoint *checkpoint = nullptr;
     rocksdb::Status s = rocksdb::Checkpoint::Create(storage->db_, &checkpoint);
     if (!s.ok()) {
       LOG(WARNING) << "Fail to create checkpoint, error:" << s.ToString();
@@ -766,7 +773,7 @@ Status Storage::ReplDataManager::GetFullReplDataInfo(Storage *storage, std::stri
   // Get checkpoint file list
   std::vector<std::string> result;
   storage->env_->GetChildren(data_files_dir, &result);
-  for (auto f : result) {
+  for (const auto &f : result) {
     if (f == "." || f == "..") continue;
     files->append(f);
     files->push_back(',');
@@ -775,12 +782,12 @@ Status Storage::ReplDataManager::GetFullReplDataInfo(Storage *storage, std::stri
   return Status::OK();
 }
 
-bool Storage::ExistCheckpoint(void) {
+bool Storage::ExistCheckpoint() {
   std::lock_guard<std::mutex> lg(checkpoint_mu_);
   return env_->FileExists(config_->checkpoint_dir).ok();
 }
 
-bool Storage::ExistSyncCheckpoint(void) { return env_->FileExists(config_->sync_checkpoint_dir).ok(); }
+bool Storage::ExistSyncCheckpoint() { return env_->FileExists(config_->sync_checkpoint_dir).ok(); }
 
 Status Storage::ReplDataManager::CleanInvalidFiles(Storage *storage, const std::string &dir,
                                                    std::vector<std::string> valid_files) {
@@ -790,7 +797,7 @@ Status Storage::ReplDataManager::CleanInvalidFiles(Storage *storage, const std::
 
   std::vector<std::string> tmp_files, files;
   storage->env_->GetChildren(dir, &tmp_files);
-  for (auto file : tmp_files) {
+  for (const auto &file : tmp_files) {
     if (file == "." || file == "..") continue;
     files.push_back(file);
   }
@@ -832,10 +839,8 @@ int Storage::ReplDataManager::OpenDataFile(Storage *storage, const std::string &
   return rv;
 }
 
-Storage::ReplDataManager::MetaInfo Storage::ReplDataManager::ParseMetaAndSave(Storage *storage,
-                                                                              rocksdb::BackupID meta_id,
-                                                                              evbuffer *evbuf) {
-  Storage::ReplDataManager::MetaInfo meta;
+Status Storage::ReplDataManager::ParseMetaAndSave(Storage *storage, rocksdb::BackupID meta_id, evbuffer *evbuf,
+                                                  Storage::ReplDataManager::MetaInfo *meta) {
   auto meta_file = "meta/" + std::to_string(meta_id);
   DLOG(INFO) << "[meta] id: " << meta_id;
 
@@ -848,16 +853,16 @@ Storage::ReplDataManager::MetaInfo Storage::ReplDataManager::ParseMetaAndSave(St
   // timestamp;
   UniqueEvbufReadln line(evbuf, EVBUFFER_EOL_LF);
   DLOG(INFO) << "[meta] timestamp: " << line.get();
-  meta.timestamp = std::strtoll(line.get(), nullptr, 10);
+  meta->timestamp = std::strtoll(line.get(), nullptr, 10);
   // sequence
   line = UniqueEvbufReadln(evbuf, EVBUFFER_EOL_LF);
   DLOG(INFO) << "[meta] seq:" << line.get();
-  meta.seq = std::strtoull(line.get(), nullptr, 10);
+  meta->seq = std::strtoull(line.get(), nullptr, 10);
   // optional metadata
   line = UniqueEvbufReadln(evbuf, EVBUFFER_EOL_LF);
   if (strncmp(line.get(), "metadata", 8) == 0) {
     DLOG(INFO) << "[meta] meta: " << line.get();
-    meta.meta_data = std::string(line.get(), line.length);
+    meta->meta_data = std::string(line.get(), line.length);
     line = UniqueEvbufReadln(evbuf, EVBUFFER_EOL_LF);
   }
   DLOG(INFO) << "[meta] file count: " << line.get();
@@ -875,10 +880,10 @@ Storage::ReplDataManager::MetaInfo Storage::ReplDataManager::ParseMetaAndSave(St
     while (*(cptr++) != ' ') {
     }
     auto crc32 = std::strtoul(cptr, nullptr, 10);
-    meta.files.emplace_back(filename, crc32);
+    meta->files.emplace_back(filename, crc32);
   }
-  SwapTmpFile(storage, storage->config_->backup_sync_dir, meta_file);
-  return meta;
+
+  return SwapTmpFile(storage, storage->config_->backup_sync_dir, meta_file);
 }
 
 Status MkdirRecursively(rocksdb::Env *env, const std::string &dir) {
@@ -943,11 +948,10 @@ bool Storage::ReplDataManager::FileExists(Storage *storage, const std::string &d
   s = storage->env_->NewSequentialFile(file_path, &src_file, soptions);
   if (!s.ok()) return false;
 
-  uint64_t size;
+  uint64_t size = 0;
   s = storage->env_->GetFileSize(file_path, &size);
   if (!s.ok()) return false;
-  std::unique_ptr<rocksdb::SequentialFileWrapper> src_reader;
-  src_reader.reset(new rocksdb::SequentialFileWrapper(src_file.get()));
+  auto src_reader = std::make_unique<rocksdb::SequentialFileWrapper>(src_file.get());
 
   char buffer[4096];
   Slice slice;
