@@ -121,8 +121,8 @@ rocksdb::Status Bitmap::GetString(const Slice &user_key, const uint32_t max_btos
   rocksdb::ReadOptions read_options;
   LatestSnapShot ss(db_);
   read_options.snapshot = ss.GetSnapShot();
-  read_options.fill_cache = false;
-  uint32_t frag_index, valid_size;
+  storage_->SetReadOptions(read_options);
+  uint32_t frag_index = 0, valid_size = 0;
 
   auto iter = DBUtil::UniqueIterator(db_, read_options);
   for (iter->Seek(prefix_key); iter->Valid() && iter->key().starts_with(prefix_key); iter->Next()) {
@@ -165,7 +165,7 @@ rocksdb::Status Bitmap::GetString(const Slice &user_key, const uint32_t max_btos
         0x3F, 0xBF, 0x7F, 0xFF};
     for (uint32_t i = 0; i < valid_size; i++) {
       if (!fragment[i]) continue;
-      fragment[i] = swap_table[static_cast<uint8_t>(fragment[i])];
+      fragment[i] = static_cast<char>(swap_table[static_cast<uint8_t>(fragment[i])]);
       ;
     }
     value->replace(frag_index, valid_size, fragment.data(), valid_size);
@@ -198,7 +198,7 @@ rocksdb::Status Bitmap::SetBit(const Slice &user_key, uint32_t offset, bool new_
   uint32_t used_size = index + byte_index + 1;
   uint32_t bitmap_size = std::max(used_size, metadata.size);
   if (byte_index >= value.size()) {  // expand the bitmap
-    size_t expand_size;
+    size_t expand_size = 0;
     if (byte_index >= value.size() * 2) {
       expand_size = byte_index - value.size() + 1;
     } else if (value.size() * 2 > kBitmapSegmentBytes) {
@@ -211,9 +211,9 @@ rocksdb::Status Bitmap::SetBit(const Slice &user_key, uint32_t offset, bool new_
   uint32_t bit_offset = offset % 8;
   *old_bit = (value[byte_index] & (1 << bit_offset)) != 0;
   if (new_bit) {
-    value[byte_index] |= 1 << bit_offset;
+    value[byte_index] = static_cast<char>(value[byte_index] | (1 << bit_offset));
   } else {
-    value[byte_index] &= ~(1 << bit_offset);
+    value[byte_index] = static_cast<char>(value[byte_index] & (~(1 << bit_offset)));
   }
   rocksdb::WriteBatch batch;
   WriteBatchLogData log_data(kRedisBitmap, {std::to_string(kRedisCmdSetBit), std::to_string(offset)});
@@ -247,8 +247,8 @@ rocksdb::Status Bitmap::BitCount(const Slice &user_key, int64_t start, int64_t s
   if (stop > static_cast<int64_t>(metadata.size)) stop = metadata.size;
   if (start < 0 || stop <= 0 || start >= stop) return rocksdb::Status::OK();
 
-  uint32_t u_start = static_cast<uint32_t>(start);
-  uint32_t u_stop = static_cast<uint32_t>(stop);
+  auto u_start = static_cast<uint32_t>(start);
+  auto u_stop = static_cast<uint32_t>(stop);
 
   LatestSnapShot ss(db_);
   rocksdb::ReadOptions read_options;
@@ -297,8 +297,8 @@ rocksdb::Status Bitmap::BitPos(const Slice &user_key, bool bit, int64_t start, i
     *pos = -1;
     return rocksdb::Status::OK();
   }
-  uint32_t u_start = static_cast<uint32_t>(start);
-  uint32_t u_stop = static_cast<uint32_t>(stop);
+  auto u_start = static_cast<uint32_t>(start);
+  auto u_stop = static_cast<uint32_t>(stop);
 
   auto bitPosInByte = [](char byte, bool bit) -> int {
     for (int i = 0; i < 8; i++) {
@@ -371,7 +371,7 @@ rocksdb::Status Bitmap::BitOp(BitOpFlags op_flag, const std::string &op_name, co
       return rocksdb::Status::InvalidArgument(kErrMsgWrongType);
     }
     if (metadata.size > max_size) max_size = metadata.size;
-    meta_pairs.emplace_back(std::make_pair(ns_op_key, metadata));
+    meta_pairs.emplace_back(ns_op_key, metadata);
   }
 
   rocksdb::WriteBatch batch;
@@ -388,11 +388,11 @@ rocksdb::Status Bitmap::BitOp(BitOpFlags op_flag, const std::string &op_name, co
 
   BitmapMetadata res_metadata;
   if (num_keys == op_keys.size() || op_flag != kBitOpAnd) {
-    uint64_t i, frag_numkeys = num_keys, stop_index = (max_size - 1) / kBitmapSegmentBytes;
+    uint64_t i = 0, frag_numkeys = num_keys, stop_index = (max_size - 1) / kBitmapSegmentBytes;
     std::unique_ptr<unsigned char[]> frag_res(new unsigned char[kBitmapSegmentBytes]);
     uint16_t frag_maxlen = 0, frag_minlen = 0;
     std::string sub_key, fragment;
-    unsigned char output, byte;
+    unsigned char output = 0, byte = 0;
     std::vector<std::string> fragments;
 
     LatestSnapShot ss(db_);
@@ -430,7 +430,7 @@ rocksdb::Status Bitmap::BitOp(BitOpFlags op_flag, const std::string &op_name, co
 
 #ifndef USE_ALIGNED_ACCESS
         if (frag_minlen >= sizeof(uint64_t) * 4 && frag_numkeys <= 16) {
-          uint64_t *lres = reinterpret_cast<uint64_t *>(frag_res.get());
+          auto *lres = reinterpret_cast<uint64_t *>(frag_res.get());
           const uint64_t *lp[16];
           for (i = 0; i < frag_numkeys; i++) {
             lp[i] = reinterpret_cast<const uint64_t *>(fragments[i].data());
@@ -536,7 +536,7 @@ rocksdb::Status Bitmap::BitOp(BitOpFlags op_flag, const std::string &op_name, co
   res_metadata.size = max_size;
   res_metadata.Encode(&bytes);
   batch.Put(metadata_cf_handle_, ns_key, bytes);
-  *len = max_size;
+  *len = static_cast<int64_t>(max_size);
   return storage_->Write(storage_->DefaultWriteOptions(), &batch);
 }
 
