@@ -93,7 +93,7 @@ def check_version(current: str, required: Tuple[int, int, int], prog_name: Optio
     return semver
 
 
-def build(dir: str, jobs: int, ghproxy: bool, ninja: bool, unittest: bool, compiler: str, cmake_path: str, D: List[str],
+def build(dir: str, jobs: Optional[int], ghproxy: bool, ninja: bool, unittest: bool, compiler: str, cmake_path: str, D: List[str],
           skip_build: bool) -> None:
     basedir = Path(__file__).parent.absolute()
 
@@ -127,7 +127,13 @@ def build(dir: str, jobs: int, ghproxy: bool, ninja: bool, unittest: bool, compi
     target = ["kvrocks", "kvrocks2redis"]
     if unittest:
         target.append("unittest")
-    run(cmake, "--build", ".", f"-j{jobs}", "-t", *target, verbose=True, cwd=dir)
+
+    options = ["--build", "."]
+    if jobs is not None:
+        options.append(f"-j{jobs}")
+    options += ["-t", *target]
+
+    run(cmake, *options, verbose=True, cwd=dir)
 
 
 def get_source_files() -> List[str]:
@@ -136,8 +142,8 @@ def get_source_files() -> List[str]:
         *glob("src/**/*.cc", recursive=True),
         *glob("tests/cppunit/**/*.h", recursive=True),
         *glob("tests/cppunit/**/*.cc", recursive=True),
-        *glob("tools/kvrocks2redis/**/*.h", recursive=True),
-        *glob("tools/kvrocks2redis/**/*.cc", recursive=True),
+        *glob("utils/kvrocks2redis/**/*.h", recursive=True),
+        *glob("utils/kvrocks2redis/**/*.cc", recursive=True),
     ]
 
 
@@ -166,7 +172,7 @@ def clang_format(clang_format_path: str, fix: bool = False) -> None:
     run(command, *options, *sources, verbose=True, cwd=basedir)
 
 
-def clang_tidy(dir: str, jobs: int, clang_tidy_path: str, run_clang_tidy_path: str) -> None:
+def clang_tidy(dir: str, jobs: Optional[int], clang_tidy_path: str, run_clang_tidy_path: str) -> None:
     # use the run-clang-tidy Python script provided by LLVM Clang
     run_command = find_command(run_clang_tidy_path, msg="run-clang-tidy is required")
     tidy_command = find_command(clang_tidy_path, msg="clang-tidy is required")
@@ -181,9 +187,11 @@ def clang_tidy(dir: str, jobs: int, clang_tidy_path: str, run_clang_tidy_path: s
 
     basedir = Path(__file__).parent.absolute()
 
-    options = ['-p', dir, '-clang-tidy-binary', tidy_command, f'-j{jobs}']
+    options = ['-p', dir, '-clang-tidy-binary', tidy_command]
+    if jobs is not None:
+        options.append(f'-j{jobs}')
 
-    run(run_command, *options, 'kvrocks/src/', verbose=True, cwd=basedir)
+    run(run_command, *options, 'kvrocks/src/', 'kvrocks2redis/', verbose=True, cwd=basedir)
 
 
 def golangci_lint() -> None:
@@ -204,7 +212,7 @@ def write_version(release_version: str) -> str:
     if SEMVER_REGEX.match(version) is None:
         raise RuntimeError(f"Kvrocks version should follow semver spec, got: {version}")
 
-    with open('VERSION', 'w+') as f:
+    with open('src/VERSION.txt', 'w+') as f:
         f.write(version)
 
     return version
@@ -235,46 +243,6 @@ def package_source(release_version: str) -> None:
     with open(f'{tarball}.sha512', 'w+') as f:
         f.write(payload)
 
-
-def package_fpm(package_type: str, release_version: str, dir: str, jobs: int) -> None:
-    fpm = find_command('fpm', msg=f'fpm is required for {package_type} packaging')
-
-    version = write_version(release_version)
-
-    build(dir=dir, jobs=jobs, ghproxy=False, ninja=False, unittest=False, skip_build=False, compiler='auto',
-          cmake_path='cmake', D=[])
-
-    package_dir = Path(dir) / 'package-fpm'
-    makedirs(str(package_dir), exist_ok=False)
-    makedirs(str(package_dir / 'bin'))
-    makedirs(str(package_dir / 'conf'))
-
-    basedir = Path(__file__).parent.absolute()
-
-    copyfile(str(Path(dir) / 'kvrocks'), str(package_dir / 'bin' / 'kvrocks'))
-    copyfile(str(Path(dir) / 'kvrocks2redis'), str(package_dir / 'bin' / 'kvrocks2redis'))
-    copyfile(str(basedir / 'kvrocks.conf'), str(package_dir / 'conf' / 'kvrocks.conf'))
-
-    fpm_opts = [
-        '-t', package_type,
-        '-v', version,
-        '-C', str(package_dir),
-        '-s', 'dir',
-        '--prefix', '/www/kvrocks',
-        '-n', 'kvrocks',
-        '--epoch', '7',
-        '--config-files', '/www/kvrocks/conf/kvrocks.conf',
-        '--iteration', 'release',
-        '--verbose',
-        '--category', 'kvrocks/projects',
-        '--description', 'kvrocks',
-        '--url', 'https://github.com/apache/incubator-kvrocks',
-        '--license', 'Apache-2.0'
-    ]
-
-    run(fpm, *fpm_opts, verbose=True)
-
-
 def test_cpp(dir: str, rest: List[str]) -> None:
     basedir = Path(dir).absolute()
     unittest = basedir / 'unittest'
@@ -337,7 +305,7 @@ if __name__ == '__main__':
     parser_check_tidy.set_defaults(func=clang_tidy)
     parser_check_tidy.add_argument('dir', metavar='BUILD_DIR', nargs='?', default='build',
                                    help="directory to store cmake-generated and build files")
-    parser_check_tidy.add_argument('-j', '--jobs', default=4, metavar='N', help='execute N build jobs concurrently')
+    parser_check_tidy.add_argument('-j', '--jobs', metavar='N', help='execute N build jobs concurrently')
     parser_check_tidy.add_argument('--clang-tidy-path', default='clang-tidy',
                                    help="path of clang-tidy used to check source")
     parser_check_tidy.add_argument('--run-clang-tidy-path', default='run-clang-tidy',
@@ -358,7 +326,7 @@ if __name__ == '__main__':
     )
     parser_build.add_argument('dir', metavar='BUILD_DIR', nargs='?', default='build',
                               help="directory to store cmake-generated and build files")
-    parser_build.add_argument('-j', '--jobs', default=4, metavar='N', help='execute N build jobs concurrently')
+    parser_build.add_argument('-j', '--jobs', metavar='N', help='execute N build jobs concurrently')
     parser_build.add_argument('--ghproxy', default=False, action='store_true',
                               help='use https://ghproxy.com to fetch dependencies')
     parser_build.add_argument('--ninja', default=False, action='store_true', help='use Ninja to build kvrocks')
@@ -387,19 +355,6 @@ if __name__ == '__main__':
     parser_package_source.add_argument('-v', '--release-version', required=True, metavar='VERSION',
                                        help='current releasing version')
     parser_package_source.set_defaults(func=package_source)
-    parser_package_fpm = parser_package_subparsers.add_parser(
-        'fpm',
-        description="Package built binaries to an rpm/deb package",
-        help="Package built binaries to an rpm/deb package",
-    )
-    parser_package_fpm.add_argument('-v', '--release-version', required=True, metavar='VERSION',
-                                    help='current releasing version')
-    parser_package_fpm.add_argument('-t', '--package-type', required=True, choices=('rpm', 'deb'),
-                                    help='package type for fpm to build')
-    parser_package_fpm.add_argument('dir', metavar='BUILD_DIR',
-                                    help="directory to store cmake-generated and build files")
-    parser_package_fpm.add_argument('-j', '--jobs', default=4, metavar='N', help='execute N build jobs concurrently')
-    parser_package_fpm.set_defaults(func=package_fpm)
 
     parser_test = subparsers.add_parser(
         'test',
